/*
 * MDC Desktop Client Created on 11-May-2005
 */
package org.mdcog.client.desktop.dataaccess;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.PrintStream;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.mdcog.client.desktop.gui.error.MDCInternalException;
import org.mdcog.client.desktop.model.ResourceItem;
import org.mdcog.client.desktop.model.ResourceList;
import org.mdcog.client.desktop.utils.MDCConfig;
import org.mdcog.client.styles.MDCStyle;
import org.mdcog.util.MDCWebServiceHandler;
import org.mdcog.util.transfer.MDCDataTransferer;
import org.mdcog.util.transfer.MDCXMLDataTransferer;
import org.mdcog.util.transfer.ResourceTransferHeader;
import org.mdcog.util.transfer.ResourceTransferItem;
import org.mdcog.util.transfer.TransferException;

/**
 * This Data Access Object implements the persistency methods for storing and
 * retrieving a users personal resource list library, exporting and importing
 * lists/search results and the creation of external documents in HTML or PDF.
 * 
 * @uml.dependency supplier="org.mdcog.client.styles.MDCStyle" stereotypes="Basic::Call"
 * @uml.dependency supplier="org.mdcog.util.MDCWebServiceHandler" stereotypes="Basic::Call"
 */

public class LibraryDAO
{
    /**
     * Stores a reference to the deafult number of records returned if not configured in properties
     */
    public static final String DEFAULT_MDC_WEB_SERVICE_MAX_RECORDS = "100";
    
    /**
     * Saves the entire library to the given filename. This method uses standard
     * Java Object Serialisation.
     * 
     * @param filename
     * @param library
     * @throws FileNotFoundException
     * @throws MDCInternalException
     */
    public void saveLibrary(String filename, Vector library) throws FileNotFoundException, MDCInternalException
    {
        try
        {
            // create a new file object for filename and create a new blank file
            // overwriting if it exists
            File file = new File(filename);
            file.createNewFile();
            // create the output streams for serialisation
            FileOutputStream outputFile = new FileOutputStream(file);
            ObjectOutputStream objectStream = new ObjectOutputStream(outputFile);
            // write the library to streams
            objectStream.writeObject(library);
            // close and flush buffers
            objectStream.flush();
            objectStream.close();
            outputFile.close();
        }
        catch (IOException e)
        {
            // This should not occur so write stack trace and throw internal
            // error
            e.printStackTrace();
            throw new MDCInternalException("Error saving library");
        }
    }

    /**
     * Loads the library from the given filename. This method uses standard Java
     * Object Serialisation
     * 
     * @param filename
     * @return The loaded library
     * @throws FileNotFoundException
     * @throws InvalidFileFormatException
     */
    public List loadLibrary(String filename) throws FileNotFoundException, InvalidFileFormatException
    {
        Vector library = null;
        try
        {
            // open the input file and streams
            FileInputStream inputFile = new FileInputStream(filename);
            ObjectInputStream inputStream = new ObjectInputStream(inputFile);
            // read in the object converting to a Vector
            library = (Vector) inputStream.readObject();
            // close streams
            inputStream.close();
            inputFile.close();
        }
        catch (IOException e)
        {
            // If an exception is thrown then we assume the file format is
            // invalid
            // so log the error and throw invalid format exception
            e.printStackTrace();
            throw new InvalidFileFormatException(e);
        }
        catch (ClassNotFoundException e)
        {
            // If an exception is thrown then we assume the file format is
            // invalid so log the error and throw invalid format exception
            e.printStackTrace();
            throw new InvalidFileFormatException(e);
        }
        // return the loaded library
        return (List) library;
    }

    /**
     * Parses the import file to extract out the resource items within it. This
     * method expects the file to be a valid XML file.
     * 
     * @param filename The filename to open
     * @return List of the resources found
     * @throws FileNotFoundException
     * @throws InvalidFileFormatException
     */
    public List importResources(String filename) throws FileNotFoundException, InvalidFileFormatException
    {
        Vector resources = new Vector();
        StringBuffer xml = new StringBuffer();

        try
        {
            // open the file and extract the string
            BufferedReader in = new BufferedReader(new FileReader(filename));
            String str;
            // loop round reading each line into buffer
            while ((str = in.readLine()) != null)
            {
                xml.append(str);
            }
            // close the reader
            in.close();

            // create the transferer to parse out the resource items
            MDCDataTransferer transferer = new MDCXMLDataTransferer();
            // load and parse the data
            transferer.loadList(xml.toString());

            // get iterator of resource items
            Iterator iter = transferer.getResourceIterator();
            ResourceItem resourceItem = null;
            // loop round the retrieved items creating resource items
            while (iter.hasNext())
            {
                // create new objects and get next transfer object
                resourceItem = new ResourceItem((ResourceTransferItem) iter.next());
                // add the resource item
                resources.add(resourceItem);
            }
        }
        catch (IOException e)

        {
            // If an exception is thrown then we assume the file format is
            // invalid so log the error and throw invalid format exception
            e.printStackTrace();
            throw new InvalidFileFormatException(e);
        }
        catch (TransferException e)
        {
            // If an exception is thrown then we assume the file format is
            // invalid so log the error and throw invalid format exception
            e.printStackTrace();
            throw new InvalidFileFormatException(e);
        }
        return resources;
    }

    /**
     * Export all the ResourceList to the file in MODS format
     * 
     * @param filename the file to export to
     * @param entry the list of resources to export
     * @throws FileNotFoundException
     * @throws MDCInternalException
     */
    public void exportAllResourcesToModsFile(String filename, ResourceList entry) throws FileNotFoundException,
            MDCInternalException
    {
        try
        {
            // create the transferer to parse out the resource items
            MDCDataTransferer transferer = createTransfererFromResourceList(entry);
            // we have now constructed all the information so convert it to mods
            // format
            // we have now constructed all the information so convert it to mods
            // format and write the created xml to the file
            saveToFile(filename, transferer.toMODS());
        }
        catch (TransferException e)
        {
            e.printStackTrace();
            throw new MDCInternalException("Error converting to MODS");
        }
    }

    /**
     * Export all the ResourceList to the file in IMS RLI format
     * 
     * @param filename the file to export to
     * @param entry the list of resources to export
     * @throws FileNotFoundException
     * @throws MDCInternalException
     */
    public void exportAllResourcesToIMSFile(String filename, ResourceList entry) throws FileNotFoundException,
            MDCInternalException
    {
        try
        {
            // create the transferer to parse out the resource items
            MDCDataTransferer transferer = createTransfererFromResourceList(entry);
            // we have now constructed all the information so convert it to mods
            // format and write the created xml to the file
            saveToFile(filename, transferer.toRLI());
        }
        catch (TransferException e)
        {
            e.printStackTrace();
            throw new MDCInternalException("Error converting to RLI");
        }
    }

    /**
     * This method constructs a transferer and initialises it with the resource
     * list information
     * 
     * @param entry The ResourceList to load into transferer
     * @return The transfer initialised with the resource list data ready for
     *         exporting
     * @throws MDCInternalException
     */
    private MDCDataTransferer createTransfererFromResourceList(ResourceList entry) throws MDCInternalException
    {
        // create the transferer to parse out the resource items
        MDCDataTransferer transferer = new MDCXMLDataTransferer();
        try
        {

            // create and populate the header with the data from the entry
            ResourceTransferHeader header = new ResourceTransferHeader();
            header.setField(ResourceTransferHeader.AUTHOR, entry.getAuthor());
            header.setField(ResourceTransferHeader.OWNER, entry.getOwner());
            header.setField(ResourceTransferHeader.TITLE, entry.getTitle());
            header.setField(ResourceTransferHeader.DATE_CREATED, entry.getCreationDate());
            header.setField(ResourceTransferHeader.ANNOTATION, entry.getAnnotation());
            // update the raw xml from the fields
            header.updateRawXMLFromFields();
            // create a new empty list for the header
            transferer.createEmptyList(header);

            ResourceTransferItem transferItem = null;
            ResourceItem resourceItem = null;
            // create transfer and add resource items
            Iterator iter = entry.getResourceItems().iterator();
            while (iter.hasNext())
            {
                // get the resource item to process and create a new
                // transferitem
                resourceItem = (ResourceItem) iter.next();
                transferItem = new ResourceTransferItem();
                // We have to load the original XML first as this will
                // prepopulate fields with old values
                if (resourceItem.getOriginalXML().length() > 0)
                {
                    // store the orignal xml back
                    transferItem.setRawXml(resourceItem.getOriginalXML());
                }
                // set all the data back into fields
                transferItem.setField(ResourceTransferItem.AUTHOR, resourceItem.getAuthor());
                transferItem.setField(ResourceTransferItem.TITLE, resourceItem.getTitle());
                transferItem.setField(ResourceTransferItem.TYPE, resourceItem.getType());
                transferItem.setField(ResourceTransferItem.ABSTRACT, resourceItem.getAbstractStr());
                transferItem.setField(ResourceTransferItem.ANNOTATION, resourceItem.getAnnotation());
                transferItem.setField(ResourceTransferItem.WEB_URL, resourceItem.getWebURL());
                // this will update the original xml with the field data
                transferItem.updateRawXMLFromFields();
                // add the transfer item to the transferer
                transferer.addResource(transferItem);
            }
        }
        catch (TransferException e)
        {
            e.printStackTrace();
            throw new MDCInternalException("Error creating transfer data");
        }
        return transferer;
    }

    /**
     * Saves the output to the filename specified overighting it if it exists
     * 
     * @param filename The file to write xml to
     * @param data The data to write
     * @throws FileNotFoundException
     * @throws MDCInternalException
     */
    private void saveToFile(String filename, String data) throws FileNotFoundException, MDCInternalException
    {
        try
        {
            // create a new file object for filename and create a new blank file
            // overwriting if it exists
            File file = new File(filename);
            file.createNewFile();
            // create the output streams for serialisation
            FileOutputStream outputFile = new FileOutputStream(file);
            PrintStream printStream = new PrintStream(outputFile);
            printStream.print(data);
            // close and flush buffers
            printStream.flush();
            printStream.close();
            outputFile.close();
        }
        catch (IOException e)
        {
            // This should not occur so write stack trace and throw internal
            // error
            e.printStackTrace();
            throw new MDCInternalException("Error saving to file");
        }
    }

    /**
     * Create the document for the given style
     * 
     * @param filename the file to create document in
     * @param style The style to apply
     * @param entry the list of resources to use
     * @param asPDF true if the output is to be converted to pdf
     * @throws FileNotFoundException
     * @throws MDCInternalException
     */
    public void applyStyle(String filename, MDCStyle style, ResourceList entry, boolean asPDF) throws FileNotFoundException,
            MDCInternalException
    {
        try
        {
            // create the transferer to parse out the resource items
            MDCDataTransferer transferer = createTransfererFromResourceList(entry);
            // convert to mods string
            String mods = transferer.toMODS();
            if (asPDF)
            {
                // create a new file object for filename and create a new blank
                // file
                // overwriting if it exists
                File file = new File(filename);
                file.createNewFile();

                // Setup output Stream
                OutputStream out = new java.io.FileOutputStream(file);
                out = new java.io.BufferedOutputStream(out);

                try
                {
                    // apply the style and convert result to pdf format
                    style.toPDF(style.applyStyle(mods), out);
                    out.flush();
                }
                finally
                {
                    // make sure we always close the stream
                    out.close();
                }
            }
            else
            {
                // just save to file
                saveToFile(filename, style.applyStyle(mods));
            }
        }
        catch (IOException e)
        {
            e.printStackTrace();
            throw new MDCInternalException("Error writing style to file");
        }
        catch (TransferException e)
        {
            e.printStackTrace();
            throw new MDCInternalException("Error applying style");
        }

    }
    
    /**
     * Searches MDC using the search text to return a collection of Resource Items.
     * @param searchText The string to search under
     * @return List of the resources found
     * @throws MDCInternalException
     */
    public List searchMDC(String searchText) throws MDCInternalException
    {
        Vector resources = new Vector();
        // get web service configuration information
        String url = MDCConfig.getInstance().getProperty(MDCConfig.MDC_WEB_SERVICE,"");
        int maxRecs = Integer.parseInt(MDCConfig.getInstance().getProperty(MDCConfig.MDC_WEB_SERVICE_MAX_RECORDS,DEFAULT_MDC_WEB_SERVICE_MAX_RECORDS));
        // create and call web service
        MDCWebServiceHandler webService = new MDCWebServiceHandler(url);
        List modsRecords = webService.doSearch(searchText,1,maxRecs);
        // get an iterator round results
        Iterator iterator = modsRecords.iterator();
        
        try
        {
            while(iterator.hasNext())
            {
                // get the xml for the mods record out
                ResourceTransferItem item = new ResourceTransferItem();
                item.setRawXml((String) iterator.next());
                // create resource item from transfer item
                ResourceItem resource = new ResourceItem(item);
                // add to return list
                resources.add(resource);
            }
        }
        catch (TransferException e)
        {
            e.printStackTrace();
            throw new MDCInternalException("Invalid format in serach results");
        }
        
        return resources;
    }
}
