/*
 * MDC Desktop Client Created on 22-Apr-2005
 */
package org.mdcog.client.desktop.gui.controllers;

import java.io.FileNotFoundException;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.ButtonGroup;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.TableModel;

import org.mdcog.client.desktop.gui.EditAnnotationDialog;
import org.mdcog.client.desktop.gui.NewResourceItemDialog;
import org.mdcog.client.desktop.gui.OpenLocalURLDialog;
import org.mdcog.client.desktop.gui.error.MDCInternalException;
import org.mdcog.client.desktop.gui.error.MDCMessageBox;
import org.mdcog.client.desktop.gui.models.MDCCreateListHTMLFilter;
import org.mdcog.client.desktop.gui.models.MDCCreateListPDFFilter;
import org.mdcog.client.desktop.gui.models.MDCExportFileFilter;
import org.mdcog.client.desktop.gui.models.ResourceItemTableFormat;
import org.mdcog.client.desktop.gui.models.ResourceListComparator;
import org.mdcog.client.desktop.model.MDCLibraryModel;
import org.mdcog.client.desktop.model.ResourceItem;
import org.mdcog.client.desktop.model.ResourceList;
import org.mdcog.client.desktop.utils.BrowserControl;
import org.mdcog.client.styles.MDCStyle;

import ca.odell.glazedlists.BasicEventList;
import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.SortedList;
import ca.odell.glazedlists.swing.EventTableModel;
import ca.odell.glazedlists.swing.TextFilterList;

/**
 * This controller controls the create list dialogue.It makes use of the Glazed
 * Lists API to provide grid sorting and text filtering
 * 
 * @see <a href="http://publicobject.com/glazedlists">Glazed Lists HomePage </a>
 */
public class CreateListDialogController extends Controller implements ResourceListHeaderControllerInterface,
        ResourceItemButtonControllerInterface
{

    /**
     * Stores a reference to the parent frame for popups
     */
    private JFrame parentFrame = null;

    /**
     * Used to identify radio button selections between the dialog and
     * controller
     */
    public final static String EXPORT_MODS = "EXPORT_MODS";

    /**
     * Used to identify radio button selections between the dialog and
     * controller
     */
    public final static String EXPORT_IMS = "SEARCH_AUTHOR";

    /**
     * Used to identify radio button selections between the dialog and
     * controller
     */
    public final static String STYLE_HTML = "STYLE_HTML";

    /**
     * Used to identify radio button selections between the dialog and
     * controller
     */
    public final static String STYLE_PDF = "STYLE_PDF";

    /**
     * Stores a reference to the library resource list entry that is the model
     * for this dialog
     * 
     * @uml.property name="libraryResourceListEntry"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private ResourceList libraryResourceListEntry = null;

    /**
     * Stores a reference to the export options radio button group that defines
     * mutual exclusivity between radio buttons
     */
    private ButtonGroup exportOptionsButtons = null;

    /**
     * Stores a reference to the format options radio button group that defines
     * mutual exclusivity between radio buttons
     */
    private ButtonGroup formatOptionsButtons = null;

    /**
     * Stores a reference to the resource table model.
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private EventTableModel resourceTableModel = null;

    /**
     * Stores a reference to the list of resources to be displayed on the grid
     * this is the base list and screen updates must be reflected here
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private EventList resourceEventList = null;

    /**
     * Stores a reference to the list of base resources in a sorted order
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private SortedList sortedResources = null;

    /**
     * Stores a reference to the base resources in a text filtered order
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private TextFilterList textFilteredResourceItems = null;

    /**
     * Constructor that takes the entries and creates all the sorted and text
     * filtured lists for the display
     * 
     * @param entry The model for the dialog
     * @param parent The frame to center message boxes on
     */
    public CreateListDialogController(ResourceList entry, JFrame parent)
    {
        // store the parent frame for message boxes
        parentFrame = parent;
        // store the entry we are manipulating
        this.libraryResourceListEntry = entry;
        // create the base list of resource items to display
        resourceEventList = new BasicEventList();
        resourceEventList.addAll(libraryResourceListEntry.getResourceItems());
        // create a sorted list of the added resources
        sortedResources = new SortedList(resourceEventList, new ResourceListComparator());
        // create the table format
        ResourceItemTableFormat tableFormat = new ResourceItemTableFormat(this);
        // create a filtered list of the sorted resources
        textFilteredResourceItems = new TextFilterList(sortedResources, tableFormat);
        // store the filter text field on the table format so it can force a
        // refilter when columns removed or added
        tableFormat.setFilterTextField(textFilteredResourceItems.getFilterEdit());
        // store the filter text field on the table format so it can force a
        // refilter when columns removed or added
        tableFormat.setFilterTextField(textFilteredResourceItems.getFilterEdit());
        // create the Resource Grid table model using the filtered list
        resourceTableModel = new EventTableModel(textFilteredResourceItems, tableFormat);
    }

    /**
     * Cleans up internal structure of this controller. Provided to support
     * glazed list requirements
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    public void dispose()
    {
        // dispose of the glazed lists to help garbage collection
        textFilteredResourceItems.dispose();
        sortedResources.dispose();
    }

    /**
     * Store the button group for detecting export options
     * 
     * @param buttonGroup
     */
    public void setExportOptionsButtonGroup(ButtonGroup buttonGroup)
    {
        exportOptionsButtons = buttonGroup;
    }

    /**
     * Store the button group for detecting format options
     * 
     * @param buttonGroup
     */
    public void setFormatOptionsButtonGroup(ButtonGroup buttonGroup)
    {
        formatOptionsButtons = buttonGroup;
    }

    /**
     * Gets the resource table model for the resources grid
     * 
     * @return TableModel The table model for the table to use
     * @uml.property name="resourceTableModel"
     */
    public TableModel getResourceTableModel()
    {
        return resourceTableModel;
    }

    /**
     * Returns the list of sorted items for the table comparator
     * 
     * @return the current list of sorted items
     */
    public SortedList getSortedResourceItems()
    {
        return sortedResources;
    }

    /**
     * This returns the filter text control assciated with the resource items
     * grid
     * 
     * @return JTextField the filter text control linked to grid
     */
    public JTextField getResourceItemsFilterTextControl()
    {
        return textFilteredResourceItems.getFilterEdit();
    }

    /**
     * This method is called to refresh the underlying grid event list if an
     * item is deleted. This will cause events to be sent to the sorted and text
     * filtered lists
     */
    private void refreshResourceGridFromModel()
    {
        // need to clear the current resourceEventList
        resourceEventList.clear();
        // add in the resources related to the newly selected list
        resourceEventList.addAll(libraryResourceListEntry.getResourceItems());
    }

    /**
     * This method deletes the specified rows from the model and then refreshes
     * the resource grid model
     * 
     * @param resourceRowsAffected
     */
    public void deleteResourceItemFromList(int[] resourceRowsAffected)
    {
        // loop round rows affected
        for (int i = 0; i < resourceRowsAffected.length; i++)
        {
            // retrieve the item at the selected row and ask the library
            // resource entry to delete it
            libraryResourceListEntry.deleteResourceItem((ResourceItem) textFilteredResourceItems.get(resourceRowsAffected[i]));
        }
        // refresh the grid now the model has been updated
        refreshResourceGridFromModel();
    }

    /**
     * This method is called to open the create resource items dialogue and add
     * the returned Resource Items to the model
     */
    public void createResourceItems()
    {
        // create the pop up
        NewResourceItemDialog dialog = new NewResourceItemDialog(applicationMainFrame);
        // show the dialog
        dialog.show();

        // Get an iterator round returned results
        Iterator items = dialog.getCreatedResources().iterator();

        // loop round adding all the items
        while (items.hasNext())
        {
            libraryResourceListEntry.addNewResourceItem((ResourceItem) items.next());
        }

        // refresh the grid now the model has been updated
        refreshResourceGridFromModel();
    }

    /**
     * Update the current lists author in to the model
     * 
     * @param author
     */
    public void setResourceListAuthor(String author)
    {
        libraryResourceListEntry.setAuthor(author);
    }

    /**
     * Update the current lists owner in to the model
     * 
     * @param owner
     */
    public void setResourceListOwner(String owner)
    {
        libraryResourceListEntry.setOwner(owner);
    }

    /**
     * Update the current lists title in to the model
     * 
     * @param title
     */
    public void setResourceListTitle(String title)
    {
        libraryResourceListEntry.setTitle(title);
    }

    /**
     * get the current lists author from the model
     * 
     * @return The author
     */
    public String getResourceListAuthor()
    {
        return libraryResourceListEntry.getAuthor();
    }

    /**
     * get the current lists owner from the model
     * 
     * @return The Owner
     */
    public String getResourceListOwner()
    {
        return libraryResourceListEntry.getOwner();
    }

    /**
     * get the current lists title from the model
     * 
     * @return The title
     */
    public String getResourceListTitle()
    {
        return libraryResourceListEntry.getTitle();
    }

    /**
     * Get the currently selected lists creation date
     * 
     * @return The date
     */
    public String getResourceListCreationDate()
    {
        return libraryResourceListEntry.getCreationDate();
    }

    /**
     * Return the create list template styles
     * 
     * @return a vector of supported document styles
     */
    public Vector getCreateDocumentStyles()
    {
        try
        {
            return getMDCLibraryModel().getCreateDocumentStyles();
        }
        catch (MDCInternalException e)
        {
            MDCMessageBox.showInternalError(e, parentFrame);
        }
        return new Vector();
    }

    /**
     * Called to open the Local URL at specified Row
     * 
     * @param row
     */
    public void openLocalURL(int row)
    {
        // create the open URL dialog with the resource item model at specified
        // row
        OpenLocalURLDialog openURLDialog = new OpenLocalURLDialog(applicationMainFrame, (ResourceItem) textFilteredResourceItems
                .get(row));
        openURLDialog.show();
    }

    /**
     * Called to open the Web URL at specified Row *
     * 
     * @param row
     */
    public void openWebURL(int row)
    {
        String webURL = ((ResourceItem) textFilteredResourceItems.get(row)).getWebURL();
        // do we have a URL to open
        if (webURL.length() > 0)
        {
            // yes so open it
            if (!BrowserControl.displayURL(webURL))
            {
                MDCMessageBox.showOpenURLError(applicationMainFrame, webURL);
            }
        }
        else
        {
            // tell user no URL assigned
            MDCMessageBox.showOpenURLError(applicationMainFrame);
        }
    }

    /**
     * Called to open the Open URL at specified Row *
     * 
     * @param row
     */
    public void openOpenURL(int row)
    {
        String openURL = ((ResourceItem) textFilteredResourceItems.get(row)).getOpenURL();
        // do we have a URL to open
        if (openURL.length() > 0)
        {
            // yes so open it
            if (!BrowserControl.displayURL(openURL))
            {
                MDCMessageBox.showOpenURLError(applicationMainFrame, openURL);
            }
        }
        else
        {
            // tell user no URL assigned
            MDCMessageBox.showOpenURLError(applicationMainFrame);
        }

    }

    /**
     * Called when the annotation button clicked at specified row
     * 
     * @param row Zero based index of the row to open
     */
    public void openResourceItemAnnotation(int row)
    {
        // get the annptation for the specified row
        String annotation = ((ResourceItem) textFilteredResourceItems.get(row)).getAnnotation();
        // create the edit annotation dialog
        EditAnnotationDialog dialog = new EditAnnotationDialog(annotation, applicationMainFrame);
        dialog.show();
        // if the annotation was updated store it back onto resource item
        if (dialog.annotationUpdated())
        {
            // store the new annotation back on item
            ((ResourceItem) textFilteredResourceItems.get(row)).setAnnotation(dialog.getAnnotation());
            // notify model change
            notifyListenersOfModelChange();
        }
    }

    /**
     * Called when the annotation button clicked on resource list header
     */
    public void openResourceListAnnotation()
    {
        // create the edit annotation dialog
        EditAnnotationDialog dialog = new EditAnnotationDialog(libraryResourceListEntry.getAnnotation(), applicationMainFrame);
        dialog.show();
        // if the annotation was updated store it back onto resource list
        if (dialog.annotationUpdated())
        {
            // store the new annotation back on list
            libraryResourceListEntry.setAnnotation(dialog.getAnnotation());
            // notify model change
            notifyListenersOfModelChange();
        }
    }

    /**
     * Export the resource list to a file.
     */
    public void export()
    {
        // TODO TEMPORY CODE NEEDS REMOVING WHEN ADD RLI SUPPORT
        if (exportOptionsButtons.getSelection().getActionCommand().equals(EXPORT_IMS))
        {
            // currently not supported
            JOptionPane.showMessageDialog(parentFrame, "Exporting to IMS RLI not yet supported", "Not yet available",
                    JOptionPane.WARNING_MESSAGE);
            return;
        }

        if (!libraryResourceListEntry.canCreateList())
        {
            MDCMessageBox.showNotEnoughDataToExportWarning(parentFrame);
            return;
        }

        // obtain the file name the user wishes to export the list to
        String newFilename = getNewFilename((FileFilter) new MDCExportFileFilter(), MDCLibraryModel.EXPORT_FILE_EXTENSION);
        if (newFilename.length() == 0)
        {
            // return as we have cancelled operation
            return;
        }

        try
        {
            // get the type of export we are doing RLI or MODS
            if (exportOptionsButtons.getSelection().getActionCommand().equals(EXPORT_IMS))
            {
                // we now have the file name to export the list under so start
                // export
                getMDCLibraryModel().exportAllResourcesToIMSFile(newFilename, libraryResourceListEntry);
            }
            else
            {
                // we now have the file name to export the list under so start
                // export
                getMDCLibraryModel().exportAllResourcesToModsFile(newFilename, libraryResourceListEntry);
            }
        }
        catch (FileNotFoundException e)
        {
            MDCMessageBox.showFileNotFound(newFilename, parentFrame);
            return;
        }
        catch (MDCInternalException e)
        {
            MDCMessageBox.showInternalError(e, parentFrame);
            return;
        }

        // report successfull export to user
        MDCMessageBox.exportComplete(newFilename, parentFrame);
    }

    /**
     * Export the resource list to a file.
     * 
     * @param style The style to apply
     */
    public void createDocument(MDCStyle style)
    {
        // do we have enough information to create a list
        if (!libraryResourceListEntry.canCreateList())
        {
            MDCMessageBox.showNotEnoughDataToCreateListWarning(parentFrame);
            return;
        }

        // get the correct filename depending upon format
        String newFilename = null;
        boolean asPDF = formatOptionsButtons.getSelection().getActionCommand().equals(STYLE_PDF);

        // Has the user selected HTML or PDF output
        if (asPDF)
        {
            // Get PDF filename
            newFilename = getNewFilename((FileFilter) new MDCCreateListPDFFilter(),
                    MDCLibraryModel.CREATE_LIST_PDF_FILE_EXTENSION);
        }
        else
        {
            // Get HTML filename
            newFilename = getNewFilename((FileFilter) new MDCCreateListHTMLFilter(),
                    MDCLibraryModel.CREATE_LIST_HTML_FILE_EXTENSION);
        }

        // only continue if filename not empty
        if (newFilename.length() == 0)
        {
            // return as we have cancelled operation
            return;
        }

        try
        {
            // apply the style to the entry
            getMDCLibraryModel().applyStyle(newFilename, style, libraryResourceListEntry, asPDF);
        }
        catch (FileNotFoundException e)
        {
            MDCMessageBox.showFileNotFound(newFilename, parentFrame);
            return;
        }
        catch (MDCInternalException e)
        {
            MDCMessageBox.showInternalError(e, parentFrame);
            return;
        }
        // open the created file up in a browser window
        BrowserControl.displayURL(newFilename);
    }

    /**
     * Opens the file dialog to get the new filename using the supplid filter
     * and extension
     * 
     * @param filter Used to filter in the file dialog
     * @param extension The extension to concatonate to the filename if missing
     * @return The new filename to use
     */
    private String getNewFilename(FileFilter filter, String extension)
    {

        String newFilename = "";
        // popup dialogue box to get new file name
        JFileChooser fileChooser = new JFileChooser();
        fileChooser.setMultiSelectionEnabled(false);
        fileChooser.setFileFilter(filter);
        // show the dialog
        int option = fileChooser.showSaveDialog(applicationMainFrame);
        // did we select a file to load
        if (option == JFileChooser.APPROVE_OPTION)
        {
            newFilename = fileChooser.getSelectedFile().getAbsolutePath();
            // add extension if missing
            if (!newFilename.endsWith(extension))
            {
                newFilename = newFilename.concat(".").concat(extension);
            }
            // now need to check if already exists and warn they are going
            // to over write file
            if (fileChooser.getSelectedFile().exists())
            {
                option = MDCMessageBox.showFileExistsWarning(newFilename, parentFrame);
                if (option == JOptionPane.NO_OPTION)
                {
                    return "";
                }
            }
        }
        else
        {
            // return as we have cancelled operation
            return "";
        }
        return newFilename;
    }
}
