/*
 * MDC Desktop Client Created on 22-Apr-2005
 */
package org.mdcog.client.desktop.gui.controllers;

import java.io.FileNotFoundException;
import java.util.List;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.TableModel;

import org.mdcog.client.desktop.dataaccess.InvalidFileFormatException;
import org.mdcog.client.desktop.gui.CreateListDialog;
import org.mdcog.client.desktop.gui.EditAnnotationDialog;
import org.mdcog.client.desktop.gui.NewResourceItemDialog;
import org.mdcog.client.desktop.gui.OpenLocalURLDialog;
import org.mdcog.client.desktop.gui.error.MDCInternalException;
import org.mdcog.client.desktop.gui.error.MDCMessageBox;
import org.mdcog.client.desktop.gui.models.MDCLibraryFileFilter;
import org.mdcog.client.desktop.gui.models.ResourceItemDragInformation;
import org.mdcog.client.desktop.gui.models.ResourceItemTableFormat;
import org.mdcog.client.desktop.gui.models.ResourceListComparator;
import org.mdcog.client.desktop.gui.models.ResourceListTableFormat;
import org.mdcog.client.desktop.gui.models.ResourceListTableModel;
import org.mdcog.client.desktop.model.MDCLibraryModel;
import org.mdcog.client.desktop.model.ResourceItem;
import org.mdcog.client.desktop.model.ResourceList;
import org.mdcog.client.desktop.utils.BrowserControl;

import ca.odell.glazedlists.BasicEventList;
import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.SortedList;
import ca.odell.glazedlists.swing.EventTableModel;
import ca.odell.glazedlists.swing.TextFilterList;

/**
 * This class controls the libary tab of the application. It makes use of the
 * Glazed Lists API to provide grid sorting and text filtering
 * 
 * @see <a href="http://publicobject.com/glazedlists">Glazed Lists HomePage </a>
 * @uml.dependency supplier="org.mdcog.client.desktop.model.ResourceList"
 *                 stereotypes="Basic::Call"
 */

public class LibraryTabController extends Controller implements ResourceListHeaderControllerInterface,
        ResourceItemButtonControllerInterface
{

    /**
     * Stores a reference to the instance to implement the singleton pattern
     * 
     * @uml.property name="instance"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private static LibraryTabController instance = new LibraryTabController();

    /**
     * Stores a reference to the table model that implements the library
     * resource list table
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     * @uml.property name="resourceListTableModel"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private ResourceListTableModel resourceListTableModel = null;

    /**
     * Stores a reference to the list of LibraryResourceListEntries to be
     * displayed on the grid
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private EventList resourceListEventList = null;

    /**
     * Stores a reference to the resource item table model for the currently
     * selected resource list.
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private EventTableModel resourceItemTableModel = null;

    /**
     * Stores a reference to the list of resources items to be displayed on the
     * grid for the currently selected resource list
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private EventList resourceItemEventList = null;

    /**
     * Stores a reference to the list of base resources in a sorted order
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private SortedList sortedResourceItems = null;

    /**
     * Stores a reference to the base resources in a text filtered order
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private TextFilterList textFilteredResourceItems = null;

    /**
     * Stores a reference to the currently selected resource list row, stored
     * for drag and drop support
     */
    private int currentlySelectedResourceListRow = 0;

    /**
     * Stores a reference to the selector that will alter the selected tab
     * 
     * @uml.property name="tabSelector"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private TabSelectorInterface tabSelector = null;

    /**
     * contructor to the master controller is private at this is a singleton.
     * The Constructor creates all the sorted and text filtured lists for the
     * display
     */
    private LibraryTabController()
    {
        // create the library resource list event list and fill it with all the
        // known resource lists from the model. By default there must be at
        // least one which is the master resource list
        resourceListEventList = new BasicEventList();
        resourceListEventList.addAll(getMDCLibraryModel().getAllResourceLists());
        // create the table model based on resourceListEventList
        resourceListTableModel = new ResourceListTableModel(resourceListEventList, new ResourceListTableFormat());

        // create the resource item event list and fill it with all the items
        // from the first element in the resourceListeEventList. By default
        // there must always be one so no defensive check required
        resourceItemEventList = new BasicEventList();
        List allResources = ((ResourceList) resourceListEventList.get(0)).getResourceItems();
        resourceItemEventList.addAll(allResources);

        // create a sorted list of the added resources items
        sortedResourceItems = new SortedList(resourceItemEventList, new ResourceListComparator());
        // create the table format
        ResourceItemTableFormat tableFormat = new ResourceItemTableFormat(this);
        // create a filtered list of the sorted resources items
        textFilteredResourceItems = new TextFilterList(sortedResourceItems, tableFormat);
        // store the filter text field on the table format so it can force a
        // refilter when columns removed or added
        tableFormat.setFilterTextField(textFilteredResourceItems.getFilterEdit());
        // create the Resource Grid table model using the filtered list
        resourceItemTableModel = new EventTableModel(textFilteredResourceItems, tableFormat);
    }

    /**
     * This Static method is used to get a reference to the LibraryTabController
     * 
     * @return LibraryTabController
     * @uml.property name="instance"
     */
    static public LibraryTabController getInstance()
    {
        return instance;
    }

    /**
     * This method stores the tab selector into the controller should it need to
     * switch the tabs programatically
     * 
     * @param selector The selector to call to change tab
     * @uml.property name="tabSelector"
     */
    public void setTabSelector(TabSelectorInterface selector)
    {
        tabSelector = selector;
    }

    /**
     * Shows a prompt to the user if the underlying model has changed since the
     * last save. If the user issues a save request then the save is initiated
     * by this method before returning to the caller
     * 
     * @return true to continue processing, false if the user cancelled
     *         operation
     */
    private boolean promptUserToSaveAnContinueIfModelChanged()
    {
        // check if the library has been changed
        if (getMDCLibraryModel().isDirty())
        {
            // prompt user if they wish to save first (YES/NO/CANCEL)
            int option = MDCMessageBox.showSaveLibraryWarning(applicationMainFrame);

            if (option == JOptionPane.YES_OPTION)
            {
                // user asked to save library first
                saveResourceLibrary(false);
            }
            else if ((option == JOptionPane.CANCEL_OPTION) || (option == JOptionPane.CLOSED_OPTION))
            {
                // user asked to cancel operation
                return false;
            }
        }
        return true;
    }

    /**
     * Creates a brand new resource library with no items. The user will be
     * prompted to save the model if it has changed since the last time it was
     * saved
     */
    public void newResourceLibrary()
    {
        // prompt user to save if model updated
        // FALSE returned if operation cancelled
        if (promptUserToSaveAnContinueIfModelChanged())
        {
            // create a new library
            getMDCLibraryModel().newLibrary();
            // clear the resource event list and repopulate it from model
            resourceListEventList.clear();
            resourceListEventList.addAll(getMDCLibraryModel().getAllResourceLists());
            // set the currently selected row to the start and refresh and
            // notify
            currentlySelectedResourceListRow = 0;
            selectedNewResourceListInMyLibary(currentlySelectedResourceListRow);

        }
    }

    /**
     * allows the user to open a new resource library file. The user will be
     * prompted to save the model if it has changed since the last time it was
     * saved
     */
    public void openResourceLibrary()
    {
        // prompt user to save if model updated
        // FALSE returned if operation cancelled
        if (promptUserToSaveAnContinueIfModelChanged())
        {
            // popup dialogue box to get the file name to open
            JFileChooser fileChooser = new JFileChooser(getMDCLibraryModel().getCurrentLibraryFileName());
            fileChooser.setMultiSelectionEnabled(false);
            fileChooser.setFileFilter((FileFilter) new MDCLibraryFileFilter());
            // show the dialog
            int option = fileChooser.showOpenDialog(applicationMainFrame);
            // did we select a file to load
            if (option == JFileChooser.APPROVE_OPTION)
            {
                String filename = fileChooser.getSelectedFile().getAbsolutePath();
                try
                {
                    // create a new library
                    getMDCLibraryModel().openLibrary(filename);
                }
                catch (FileNotFoundException e)
                {
                    MDCMessageBox.showFileNotFound(filename, applicationMainFrame);
                    return;

                }
                catch (InvalidFileFormatException e)
                {
                    MDCMessageBox.showInvalidFileFormat(filename, applicationMainFrame);
                    return;
                }
                // clear the resource event list and repopulate it from model
                resourceListEventList.clear();
                resourceListEventList.addAll(getMDCLibraryModel().getAllResourceLists());
                // set the currently selected row to the start and refresh and
                // notify
                currentlySelectedResourceListRow = 0;
                selectedNewResourceListInMyLibary(currentlySelectedResourceListRow);

            }
        }

    }

    /**
     * Returns a boolean to indicate if the model can be saved. This can be used
     * to enable menu items
     * 
     * @return true if the model has changed and hence can be changed
     */
    public boolean canSave()
    {
        return getMDCLibraryModel().isDirty();
    }

    /**
     * This method should be called when the application is about to close. It
     * will prompt the user to save the model first before the application. If
     * the user cancels any part of the save process then the application will
     * still terminate and any changes will be loast
     */
    public void applicationTerminating()
    {
        // is the model out of date and could be saved
        if (canSave())
        {
            int option = MDCMessageBox.showTerminateWithOutSavingWarning(applicationMainFrame);
            if (option == JOptionPane.YES_OPTION)
            {
                saveResourceLibrary(false);
            }
        }
    }

    /**
     * Saves the current resource library to disk under its current filename. If
     * the library has not been saved before or the save is flagged as a SaveAs
     * operation the user will be prompted to select a new filename and also
     * prompted if the file exists before it is over written.
     * 
     * @param underNewFileName If true the save is treated as SaveAs and the
     *        user will be prompted for a new file name
     */
    public void saveResourceLibrary(boolean underNewFileName)
    {
        String newFilename = null;
        // get a new file name if underNewFileName is true or model does not
        // currently have a default name
        if ((underNewFileName) || (getMDCLibraryModel().getCurrentLibraryFileName().length() <= 0))
        {
            // popup dialogue box to get new file name
            JFileChooser fileChooser = new JFileChooser(getMDCLibraryModel().getCurrentLibraryFileName());
            fileChooser.setMultiSelectionEnabled(false);
            fileChooser.setFileFilter((FileFilter) new MDCLibraryFileFilter());
            // show the dialog
            int option = fileChooser.showSaveDialog(applicationMainFrame);
            // did we select a file to load
            if (option == JFileChooser.APPROVE_OPTION)
            {
                newFilename = fileChooser.getSelectedFile().getAbsolutePath();
                // add extension if missing
                if (!newFilename.endsWith(MDCLibraryModel.LIBRARY_FILE_EXTENSION))
                {
                    newFilename = newFilename.concat(".").concat(MDCLibraryModel.LIBRARY_FILE_EXTENSION);
                }
                // now need to check if already exists and warn they are going
                // to over write file
                if (fileChooser.getSelectedFile().exists())
                {
                    option = MDCMessageBox.showFileExistsWarning(newFilename, applicationMainFrame);
                    if (option == JOptionPane.NO_OPTION)
                    {
                        return;
                    }
                }
            }
            else
            {
                // return as we have cancelled operation
                return;
            }
        }
        try
        {
            // ask the model to save passing value of new filename
            getMDCLibraryModel().saveLibrary(newFilename);
            // notify the listsners that the model has changed
            notifyListenersOfModelChange();
        }
        catch (FileNotFoundException e)
        {
            MDCMessageBox.showFileNotFound(newFilename, applicationMainFrame);
            return;
        }
        catch (MDCInternalException e)
        {
            MDCMessageBox.showInternalError(e, applicationMainFrame);
            return;
        }
    }

    /**
     * Returns the index of the row that is currently selected in the
     * MyResourceList grid.
     * 
     * @return zero based index of selected row
     * @uml.property name="currentlySelectedResourceListRow"
     */
    public int getCurrentlySelectedResourceListRow()
    {
        return currentlySelectedResourceListRow;
    }

    /**
     * Return the TableModel that controls the Resource Lists table.
     * 
     * @return TableModel
     * @uml.property name="resourceListTableModel"
     */
    public TableModel getResourceListTableModel()
    {
        return resourceListTableModel;
    }

    /**
     * Return the TableModel that controls the Resource Items table.
     * 
     * @return TableModel
     * @uml.property name="resourceItemTableModel"
     */
    public TableModel getResourceItemTableModel()
    {
        return resourceItemTableModel;
    }

    /**
     * This returns the filter text control assciated with the resource items
     * grid so that it can be displayed on the screen.
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     * @return JTextField the filter text control linked to resource item grid
     */
    public JTextField getResourceItemsFilterTextControl()
    {
        return textFilteredResourceItems.getFilterEdit();
    }

    /**
     * Returns the list of sorted items for the Glazed Lists table comparator
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     * @return the current list of sorted items
     * @uml.property name="sortedResourceItems"
     */
    public SortedList getSortedResourceItems()
    {
        return sortedResourceItems;
    }

    /**
     * Refreshes the resource event list from the model
     */
    private void refreshResourceListEventList()
    {
        resourceListEventList.clear();
        resourceListEventList.addAll(getMDCLibraryModel().getAllResourceLists());
    }

    /**
     * Refreshes the resource item event list with the resource items in the
     * specified ResourceList. This is retrieved by getting the ResourceList
     * from the resource list event list at the given row index
     * 
     * @param row
     */
    private void refreshResourceItemEventList(int row)
    {
        // need to clear the current resourceEventList
        resourceItemEventList.clear();
        // add in the resources related to the newly selected list
        List resources = ((ResourceList) resourceListEventList.get(row)).getResourceItems();
        resourceItemEventList.addAll(resources);

    }

    /**
     * This method is called when the user selects a resource list in the grid.
     * It is responsible for updating the base resource event list behind the
     * resource grid with the selected lists resource items and notifying any
     * listeners that the underlying model has changed so that controls that do
     * not automatically repaint themselves get updated
     * 
     * @param row The row selected
     */
    public void selectedNewResourceListInMyLibary(int row)
    {
        // Am aware of performance implications here but as we are only dealing
        // with references and glazed lists is optimised to cater for these
        // scenarios will stay with simplified approach. This may need to be
        // addressed if we have issues when the lists are big. However, wanted
        // to avoid complications of creating multiple cached event lists as
        // this would mean alot of extra overhead on screen painting as the
        // event lists is heavily coupled with the JTable display, sorting and
        // text filtering mechanisum.

        currentlySelectedResourceListRow = row;
        refreshResourceItemEventList(row);
        notifyListenersOfModelChange();
    }

    /**
     * This method can be called to create a new personal list. The index of the
     * new row is returned so that the display can select it if it wishes.
     * 
     * @return Zero based index of the new row
     */
    public int addNewResourceListToMyLibary()
    {
        // add the new library resource with no items to library
        getMDCLibraryModel().addNewResourceListToMyLibrary(null);
        // refresh the entire resource event list
        refreshResourceListEventList();

        // notify the listsners that the model has changed
        notifyListenersOfModelChange();
        // return the last index position as it has been added to the end
        return getMDCLibraryModel().getNumberOfResourceListsInMyLibrary() - 1;
    }

    /**
     * This method creates a new resource list automatically adding the provided
     * resource item models in to it.
     * 
     * @param resourceItems An array of resourceItem models to add
     */
    public void addNewResourceListWithItemsToMyLibary(ResourceItem[] resourceItems)
    {
        // add the new library resource with no items to library
        getMDCLibraryModel().addNewResourceListToMyLibrary(resourceItems);
        // refresh the entire resource event list
        refreshResourceListEventList();

        // set the currently selected row to the new resource list and refresh
        currentlySelectedResourceListRow = getMDCLibraryModel().getNumberOfResourceListsInMyLibrary() - 1;
        selectedNewResourceListInMyLibary(currentlySelectedResourceListRow);
    }

    /**
     * This method deletes the user libray specified by the row index
     * 
     * @param row The row to be deleted
     */
    public void deleteResourceListFromMyLibary(int row)
    {
        // get the resource list entry to delete from the resource event list
        ResourceList entry = (ResourceList) resourceListEventList.get(row);
        getMDCLibraryModel().deleteResourceListFromMyLibrary(entry);
        refreshResourceListEventList();
        // take one of currently selected row and select it
        selectedNewResourceListInMyLibary(currentlySelectedResourceListRow -1);
    }

    /**
     * This method creates a ResourceItemDragInformation object that contains
     * the specified rows for the currently selected resource list
     * 
     * @see org.mdcog.client.desktop.gui.models.ResourceItemDragInformation
     * @param rows An array of zero based indexs of resource items to add
     * @return The created ResourceItemDragInformation object
     */
    public ResourceItemDragInformation createResourceItemDragInformation(int[] rows)
    {
        ResourceItem[] items = new ResourceItem[rows.length];
        // add all the items into the array
        for (int i = 0; i < rows.length; i++)
        {
            // get the item out of the text filtered resource item
            items[i] = (ResourceItem) textFilteredResourceItems.get(rows[i]);
        }

        return new ResourceItemDragInformation(currentlySelectedResourceListRow, items);
    }

    /**
     * When an item is dragged from the resource item grid onto a resource list
     * element this method is called to add the resource item to the specified
     * row
     * 
     * @see org.mdcog.client.desktop.gui.models.ResourceItemDragInformation
     * @param resourceListRowToAddTo zero based index of resource list row to
     *        add to
     * @param dragInfo contains all the information about the item and the list
     *        it was dragged from
     */
    public void addDraggedResourceItemToResourceListMyLibary(int resourceListRowToAddTo, ResourceItemDragInformation dragInfo)
    {
        // get the entry out that we are adding items to
        ResourceList entry = (ResourceList) resourceListEventList.get(resourceListRowToAddTo);
        ResourceItem[] items = dragInfo.getResourceItems();

        // loop round adding entries
        for (int i = 0; i < items.length; i++)
        {
            // add the item, will only be addedif does not already exist
            entry.addNewResourceItem(items[i]);
        }
        // select , refresh and notify
        selectedNewResourceListInMyLibary(resourceListRowToAddTo);
    }

    /**
     * When an item is dragged from the resource item grid onto a resource list
     * as a move this method is called to remove the original resource item from
     * its list. This method only deletes if not trying to move the resource
     * item onto the same list it cam from,the all resources list and the source
     * of the move was not the master all resources list
     * 
     * @see org.mdcog.client.desktop.gui.models.ResourceItemDragInformation
     * @param dragInfo ontains all the information about the item and the list
     *        it was dragged from
     */
    public void deleteDraggedResourceItemFromResourceListFromLibary(ResourceItemDragInformation dragInfo)
    {
        // only allow the delete if the currently selected row is not all
        // resources. if it is it means the user attempted to move from personal
        // to all resources which is not allowed. Its also not allowed to move
        // to the same folder its already in so check that currently selected is
        // not the same row that it is trying to be deleted from and finally you
        // do not delete if source was all resources
        if ((currentlySelectedResourceListRow != 0) && (dragInfo.getResourceListRow() != 0)
                && (currentlySelectedResourceListRow != dragInfo.getResourceListRow()))
        {
            // get the entry out that we are adding items to
            ResourceList entry = (ResourceList) resourceListEventList.get(dragInfo.getResourceListRow());
            ResourceItem[] items = dragInfo.getResourceItems();

            // loop round deleting entries
            for (int i = 0; i < items.length; i++)
            {
                // add the item, will only be addedif does not already exist
                entry.deleteResourceItem(items[i]);
            }
            // select , refresh and notify
            selectedNewResourceListInMyLibary(currentlySelectedResourceListRow);

        }
    }

    /**
     * This method is called to copy the specified ResourceItems rows in the
     * currently selected list
     * 
     * @param rowsAffected array of zero based row indexs
     */
    public void copyResourceItemFromListInMyLibary(int[] rowsAffected)
    {
        // retrieve the entries we are copying to
        ResourceList allResourcesEntry = (ResourceList) resourceListEventList.get(0);
        ResourceList entry = (ResourceList) resourceListEventList.get(currentlySelectedResourceListRow);
        List originalItems = entry.getResourceItems();

        for (int i = 0; i < rowsAffected.length; i++)
        {
            // as only copying and adding to end indexs will remain okay
            ResourceItem resourceItem = (ResourceItem) originalItems.get(rowsAffected[i]);
            ResourceItem clonedResourceItem = (ResourceItem) resourceItem.clone();

            // add the clone to the master list
            allResourcesEntry.addNewResourceItem(clonedResourceItem);
            // are we copying from a users list
            if (entry != allResourcesEntry)
            {
                entry.addNewResourceItem(clonedResourceItem);
            }
        }
        // select , refresh and notify
        selectedNewResourceListInMyLibary(currentlySelectedResourceListRow);
    }

    /**
     * This method is called to delete the specified ResourceItems rows in the
     * currently selected list. If the currently selected row is AllResources
     * then the user will be prompted first as this will not only delete from
     * all resources but will also delete the item from any personal lists as
     * well
     * 
     * @param rowsAffected array of zero based row indexs
     */
    public void deleteResourceItemFromListInMyLibary(int[] rowsAffected)
    {
        ResourceList entry = (ResourceList) resourceListEventList.get(currentlySelectedResourceListRow);
        ResourceItem item = null;

        if (currentlySelectedResourceListRow == 0)
        {
            // pop up a warning dialogue box to make sure they want to
            // delete
            // the master and hence all the
            // references in any other resource lists
            int option = MDCMessageBox.showDeleteMasterResourceWarning(applicationMainFrame);
            // user confirmed the delete so remove it from entire library
            if (option == JOptionPane.YES_OPTION)
            {
                // iterate removing each item from entire libray
                for (int i = 0; i < rowsAffected.length; i++)
                {
                    item = (ResourceItem) textFilteredResourceItems.get(rowsAffected[i]);
                    getMDCLibraryModel().deleteResourceItemFromEntireLibrary(item);
                }
            }
        }
        else
        {
            // iterate removing item from current entry only
            for (int i = 0; i < rowsAffected.length; i++)
            {
                item = (ResourceItem) textFilteredResourceItems.get(rowsAffected[i]);
                entry.deleteResourceItem(item);
            }
        }

        // select , refresh and notify
        selectedNewResourceListInMyLibary(currentlySelectedResourceListRow);

    }

    /**
     * Opens the create list dialog with a copy of the currently selected
     * resource list data. A message will be displayed if this is attempted on a
     * list with no items.
     */
    public void createList()
    {
        // get the entry for the currently selected list
        ResourceList entry = (ResourceList) resourceListEventList.get(currentlySelectedResourceListRow);

        // before we do anything check we have resource items
        if (entry.getResourceItems().size() > 0)
        {

            // we need to clone whole library resource item now.
            ResourceList clonedEntry = (ResourceList) entry.clone();
            // create the popup dialogue
            CreateListDialog dialog = new CreateListDialog(clonedEntry, applicationMainFrame);
            // show the dilaogue.
            dialog.show();
        }
        else
        {
            // display error message to user
            MDCMessageBox.showCreateListWarning(applicationMainFrame);
        }
    }

    /**
     * This method starts the NewResourceItemDialog to allow the user to add new
     * ResourceItems to the currently selected resource list
     */
    public void createResourceItems()
    {
        // create the pop up
        NewResourceItemDialog dialog = new NewResourceItemDialog(applicationMainFrame);
        // show the dialog
        dialog.show();

        // get the items and convert to an array
        ResourceItem[] items = new ResourceItem[dialog.getCreatedResources().size()];
        items = (ResourceItem[]) dialog.getCreatedResources().toArray(items);

        // ask the model to add the resources to selected row and all resources
        ResourceList allResourcesEntry = (ResourceList) resourceListEventList.get(0);
        ResourceList entry = (ResourceList) resourceListEventList.get(currentlySelectedResourceListRow);
        for (int i = 0; i < items.length; i++)
        {
            // add to all resources
            allResourcesEntry.addNewResourceItem(items[i]);
            // add to entry if different from all resources
            if (entry != allResourcesEntry)
            {
                entry.addNewResourceItem(items[i]);
            }
        }
        // select , refresh and notify
        selectedNewResourceListInMyLibary(currentlySelectedResourceListRow);

    }

    /**
     * Update the currently selected lists author
     * 
     * @param author
     */
    public void setResourceListAuthor(String author)
    {
        ResourceList entry = (ResourceList) resourceListEventList.get(currentlySelectedResourceListRow);
        entry.setAuthor(author);
        // notify the listsners that the model has changed
        notifyListenersOfModelChange();
    }

    /**
     * Update the currently selected lists owner
     * 
     * @param owner
     */
    public void setResourceListOwner(String owner)
    {
        ResourceList entry = (ResourceList) resourceListEventList.get(currentlySelectedResourceListRow);
        entry.setOwner(owner);
        // notify the listsners that the model has changed
        notifyListenersOfModelChange();
    }

    /**
     * Update the currently selected lists title
     * 
     * @param title
     */
    public void setResourceListTitle(String title)
    {
        ResourceList entry = (ResourceList) resourceListEventList.get(currentlySelectedResourceListRow);
        entry.setTitle(title);
        // notify the listsners that the model has changed
        notifyListenersOfModelChange();
    }

    /**
     * Get the currently selected lists author
     * 
     * @return The author
     */
    public String getResourceListAuthor()
    {
        ResourceList entry = (ResourceList) resourceListEventList.get(currentlySelectedResourceListRow);
        return entry.getAuthor();
    }

    /**
     * Get the currently selected lists creation date
     * 
     * @return The date
     */
    public String getResourceListCreationDate()
    {
        ResourceList entry = (ResourceList) resourceListEventList.get(currentlySelectedResourceListRow);
        return entry.getCreationDate();
    }

    /**
     * Get the currently selected lists owner
     * 
     * @return The Owner
     */
    public String getResourceListOwner()
    {
        ResourceList entry = (ResourceList) resourceListEventList.get(currentlySelectedResourceListRow);
        return entry.getOwner();
    }

    /**
     * Get the currently selected lists title
     * 
     * @return The title
     */
    public String getResourceListTitle()
    {
        ResourceList entry = (ResourceList) resourceListEventList.get(currentlySelectedResourceListRow);
        return entry.getTitle();
    }

    /**
     * This method is called to open the local URL for the specifed resource
     * item row. This method obtains the resource item and can modify its URL
     * contents in the subsequently displayed dialog.
     * 
     * @param row Zero based index of the row to open
     */
    public void openLocalURL(int row)
    {
        ResourceItem item = (ResourceItem) textFilteredResourceItems.get(row);
        // create the open URL dialog with the resource item model at specified
        // row
        OpenLocalURLDialog openURLDialog = new OpenLocalURLDialog(applicationMainFrame, item);
        openURLDialog.show();
        // notify the listeners that the model has changed
        notifyListenersOfModelChange();
    }

    /**
     * Called to open the Web URL at specified Row.This currently uses the
     * BrowserControl from JavaWorld website that supports (windows/unix/mac)
     * and an error is displayed if this control fails to open resource
     * 
     * @see org.mdcog.client.desktop.utils.BrowserControl
     * @param row Zero based index of the row to open
     */
    public void openWebURL(int row)
    {
        String webURL = ((ResourceItem) textFilteredResourceItems.get(row)).getWebURL();
        // do we have a URL to open
        if (webURL.length() > 0)
        {
            // yes so open it
            if (!BrowserControl.displayURL(webURL))
            {
                MDCMessageBox.showOpenURLError(applicationMainFrame, webURL);
            }
        }
        else
        {
            // tell user no URL assigned
            MDCMessageBox.showOpenURLError(applicationMainFrame);
        }
    }

    /**
     * Called to open the Open URL at specified Row.This currently uses the
     * BrowserControl from JavaWorld website that supports (windows/unix/mac)
     * and an error is displayed if this control fails to open resource
     * 
     * @see org.mdcog.client.desktop.utils.BrowserControl
     * @param row Zero based index of the row to open
     */
    public void openOpenURL(int row)
    {
        String openURL = ((ResourceItem) textFilteredResourceItems.get(row)).getOpenURL();
        // do we have a URL to open
        if (openURL.length() > 0)
        {
            // yes so open it
            if (!BrowserControl.displayURL(openURL))
            {
                MDCMessageBox.showOpenURLError(applicationMainFrame, openURL);
            }
        }
        else
        {
            // tell user no URL assigned
            MDCMessageBox.showOpenURLError(applicationMainFrame);
        }

    }

    /**
     * Called when the annotation button clicked at specified row
     * 
     * @param row Zero based index of the row to open
     */
    public void openResourceItemAnnotation(int row)
    {
        // get the annptation for the specified row
        String annotation = ((ResourceItem) textFilteredResourceItems.get(row)).getAnnotation();
        // create the edit annotation dialog
        EditAnnotationDialog dialog = new EditAnnotationDialog(annotation, applicationMainFrame);
        dialog.show();
        // if the annotation was updated store it back onto resource item
        if (dialog.annotationUpdated())
        {
            // store the new annotation back on item
            ((ResourceItem) textFilteredResourceItems.get(row)).setAnnotation(dialog.getAnnotation());
            // notify model change
            notifyListenersOfModelChange();
        }
    }

    /**
     * Called when the annotation button clicked on resource list header
     */
    public void openResourceListAnnotation()
    {
        // get the current library resource entyr
        ResourceList entry = (ResourceList) resourceListEventList.get(currentlySelectedResourceListRow);

        // create the edit annotation dialog
        EditAnnotationDialog dialog = new EditAnnotationDialog(entry.getAnnotation(), applicationMainFrame);
        dialog.show();
        // if the annotation was updated store it back onto resource list
        if (dialog.annotationUpdated())
        {
            // store the new annotation back on list
            entry.setAnnotation(dialog.getAnnotation());
            // notify model change
            notifyListenersOfModelChange();
        }
    }
}