/*
 * MDC Desktop Client Created on 16-May-2005
 */
package org.mdcog.client.desktop.gui.controllers;

import java.awt.Component;
import java.io.File;
import java.net.MalformedURLException;

import javax.swing.JFileChooser;

import org.mdcog.client.desktop.gui.error.MDCMessageBox;
import org.mdcog.client.desktop.model.ResourceItem;
import org.mdcog.client.desktop.utils.BrowserControl;

/**
 * This class is the controller for the OpenLocalURLDialog
 */
public class OpenLocalURLDialogController extends Controller
{

    /**
     * Stores a reference to the string to display if no URL currently stored
     */
    private static final String NO_ASSIGNED_FILE = "No file currently assigned";

    /**
     * Stores a reference to the ResourceItem model for the dialog
     * 
     * @uml.property name="masterResourceItem"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private ResourceItem masterResourceItem = null;

    /**
     * Constructor
     * 
     * @param item the ResourceItem model
     */
    public OpenLocalURLDialogController(ResourceItem item)
    {
        masterResourceItem = item;
    }

    /**
     * Open the local URL of the resource in a browser window. This currently
     * uses the BrowserControl from JavaWorld website that supports
     * (windows/unix/mac) and an error is displayed if this control fails to
     * open resource
     * 
     * @see org.mdcog.client.desktop.utils.BrowserControl
     */
    public void openLocalURL()
    {
        if (!BrowserControl.displayURL(getURL()))
        {
            MDCMessageBox.showOpenURLError(applicationMainFrame, getURL());
        }
    }

    /**
     * Clears the assigned file from the model blanking it back to an empty
     * string
     */
    public void clearAssignedFile()
    {
        masterResourceItem.setLocalURL("");
        // notify listeners of model update
        notifyListenersOfModelChange();
    }

    /**
     * This method opens a file dialog to select a new local file to associate
     * to the resource
     * 
     * @param parent The component that the dialog should be modal to
     */
    public void assignNewFile(Component parent)
    {
        // popup dialogue box to get the file name to open
        JFileChooser fileChooser = new JFileChooser(getURL());
        fileChooser.setMultiSelectionEnabled(false);
        // show the dialog
        int option = fileChooser.showOpenDialog(parent);
        // did we select a file to assign
        if (option == JFileChooser.APPROVE_OPTION)
        {
            try
            {
                // store the URL representation of the file
                masterResourceItem.setLocalURL(fileChooser.getSelectedFile().toURL().toString());
            }
            catch (MalformedURLException e)
            {
                // should never come here as coming from file chooser
                e.printStackTrace();
            }
            // notify listeners of model update
            notifyListenersOfModelChange();
        }
    }

    /**
     * this method strips the filename from the URL and returns it to the caller
     * 
     * @return The filename in the URL
     */
    public String getFilename()
    {
        if (!canOpenURL())
        {
            return NO_ASSIGNED_FILE;
        }
        // Use File object to extract filename
        return new File(masterResourceItem.getLocalURL()).getName();

    }

    /**
     * Return the full URL
     * 
     * @return The complete URL
     */
    public String getURL()
    {
        if (!canOpenURL())
        {
            return NO_ASSIGNED_FILE;
        }
        return masterResourceItem.getLocalURL();
    }

    /**
     * Indicates if the current model has a URL hat can be opened so the display
     * can enable buttons correctly
     * 
     * @return true if a URL is currently stored
     */
    public boolean canOpenURL()
    {
        return masterResourceItem.getLocalURL().length() > 0;
    }
}
