/*
 * MDC Desktop Client Created on 03-May-2005
 */
package org.mdcog.client.desktop.gui.controllers;

import java.util.List;

import javax.swing.JTextField;
import javax.swing.table.TableModel;

import org.mdcog.client.desktop.gui.error.MDCMessageBox;
import org.mdcog.client.desktop.gui.models.ResourceItemDragInformation;
import org.mdcog.client.desktop.gui.models.ResourceListComparator;
import org.mdcog.client.desktop.gui.models.SearchResultResourceItemTableFormat;
import org.mdcog.client.desktop.model.ResourceItem;
import org.mdcog.client.desktop.utils.BrowserControl;

import ca.odell.glazedlists.BasicEventList;
import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.SortedList;
import ca.odell.glazedlists.swing.EventTableModel;
import ca.odell.glazedlists.swing.TextFilterList;

/**
 * When ever a user performs a serach or imports a new list this controller is
 * instantiated to manage the results tab that is created.It makes use of the
 * Glazed Lists API to provide grid sorting and text filtering
 * 
 * @see <a href="http://publicobject.com/glazedlists">Glazed Lists HomePage </a>
 */
public class SearchResultsTabController extends Controller implements ResourceItemButtonControllerInterface
{

    /**
     * Stores a reference to the the master search controller
     * 
     * @uml.property name="searchTabController"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private SearchTabController searchTabController = null;

    /**
     * Stores a reference to the resource item table model for the results
     * resource list on the tab.
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private EventTableModel resultsResourceTableModel = null;

    /**
     * Stores a reference to the results of the search or import. This is in
     * affect the model for the tab as all data is temporary until moved to the
     * main library
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private EventList resourceEventList = null;

    /**
     * Stores a reference to the list of base resources in a sorted order
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private SortedList sortedResources = null;

    /**
     * Stores a reference to the base resources in a text filtered order
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private TextFilterList textFilteredResourceItems = null;

    /**
     * Constructs the controller to manage the specific results tab
     * 
     * @param controller A reference to the master search controller
     * @param results The list of ResourceItems from search results or import
     */
    public SearchResultsTabController(SearchTabController controller, List results)
    {
        searchTabController = controller;
        // create the base list wheich will be empty to start
        resourceEventList = new BasicEventList();
        // store the results into the model event list
        resourceEventList.addAll(results);
        // create a sorted list of the added resources
        sortedResources = new SortedList(resourceEventList, new ResourceListComparator());
        // create the table format
        SearchResultResourceItemTableFormat tableFormat = new SearchResultResourceItemTableFormat(this);
        // create the Resource Grid table model using the filtered list
        // create a filtered list of the sorted resources
        textFilteredResourceItems = new TextFilterList(sortedResources, tableFormat);
        // store the filter text field on the table format so it can force a
        // refilter when columns removed or added
        tableFormat.setFilterTextField(textFilteredResourceItems.getFilterEdit());
        // create the table model for this tab
        resultsResourceTableModel = new EventTableModel(textFilteredResourceItems, tableFormat);
    }

    /**
     * Cleans up internal structure of this controller. Provided to support
     * glazed list requirements
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    public void closeAndCleanUp()
    {
        // remove reference to tab controller
        searchTabController = null;
        // clean up event lists so they get garbage collected
        resultsResourceTableModel.dispose();
        sortedResources.dispose();
        textFilteredResourceItems.dispose();
        resultsResourceTableModel = null;
        resourceEventList = null;
        sortedResources = null;
        textFilteredResourceItems = null;
    }

    /**
     * Gets the resource table model for the search results resources grid
     * 
     * @return TableModel The table model for the table to use
     */
    public TableModel getResourceTableModel()
    {
        return resultsResourceTableModel;
    }

    /**
     * Returns the list of sorted items for the table comparator
     * 
     * @return the current list of sorted items
     */
    public SortedList getSortedResourceItems()
    {
        return sortedResources;
    }

    /**
     * This returns the filter text control assciated with the resource items
     * grid so that it can be displayed on the screen.
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     * @return JTextField the filter text control linked to resource item grid
     */
    public JTextField getResourceItemsFilterTextControl()
    {
        return textFilteredResourceItems.getFilterEdit();
    }

    /**
     * This method copies the specified ResourceItems rows from the results to
     * the search tabs MyList grid via the search tab controller.
     * 
     * @param items An array of resource items
     */
    public void copySearchResultResourceItemsToMyList(ResourceItem[] items)
    {
        // ask the search tab controller to add the resources to MyList grid
        searchTabController.addResourceItemsToMyList(items);
    }

    /**
     * This method copies the specified ResourceItems rows from the results to
     * the search tabs MyList grid via the search tab controller.
     * 
     * @param rows An array of zero based indexs of the selected rows
     */
    public void copySearchResultResourceItemsToMyList(int[] rows)
    {
        ResourceItem[] items = new ResourceItem[rows.length];
        // add all the items into the array
        for (int i = 0; i < rows.length; i++)
        {
            // get the item out of the text filtered resource item
            items[i] = (ResourceItem) textFilteredResourceItems.get(rows[i]);
        }
        copySearchResultResourceItemsToMyList(items);
    }

    /**
     * This method creates a ResourceItemDragInformation object that contains
     * the specified rows for the currently selected resource list
     * 
     * @param rows An array of zero based indexs of resource items to add
     * @return The created ResourceItemDragInformation object
     */
    public ResourceItemDragInformation createResourceItemDragInformation(int[] rows)
    {
        ResourceItem[] items = new ResourceItem[rows.length];
        // add all the items into the array
        for (int i = 0; i < rows.length; i++)
        {
            // get the item out of the text filtered resource item
            items[i] = (ResourceItem) textFilteredResourceItems.get(rows[i]);
        }

        return new ResourceItemDragInformation(this, items);
    }

    /**
     * Delete the select rows from the results grid
     * 
     * @param rows An array of zero based indexs of the selected row
     */
    public void deleteResourceItemsFromList(int[] rows)
    {
        ResourceItem[] resourceItems = getMyListResourceItemsForRows(rows);
        //loop round and get the items to delete
        for (int i = 0; i < rows.length; i++)
        {
            textFilteredResourceItems.remove(resourceItems[i]);
        }
    }

    /**
     * Extracts the actual resource items for the specifed rows and returns them
     * in an array
     * 
     * @param rows An array of zero based indexs of the selected row
     * @return An array of resource items that correspond to rows
     */
    private ResourceItem[] getMyListResourceItemsForRows(int[] rows)
    {
        // create an array to hold resource items
        ResourceItem[] resourceItems = new ResourceItem[rows.length];

        // loop round and get the items to delete
        for (int i = 0; i < rows.length; i++)
        {
            // copy the references into array from text filtered list as this is
            // the list the indexs match against
            resourceItems[i] = (ResourceItem) textFilteredResourceItems.get(rows[i]);
        }
        return resourceItems;
    }

    /**
     * This method is called to open the local URL for the specifed resource
     * item row.This grid however does not show it so do nothing
     * 
     * @param row Zero based index of the row to open
     */
    public void openLocalURL(int row)
    {
        // button does not exist on grid so will never be called
    }

    /**
     * Called to open the Web URL at specified Row.This currently uses the
     * BrowserControl from JavaWorld website that supports (windows/unix/mac)
     * and an error is displayed if this control fails to open resource
     * 
     * @see org.mdcog.client.desktop.utils.BrowserControl
     * @param row Zero based index of the row to open
     */
    public void openWebURL(int row)
    {
        String webURL = ((ResourceItem) textFilteredResourceItems.get(row)).getWebURL();
        // do we have a URL to open
        if (webURL.length() > 0)
        {
            // yes so open it
            if (!BrowserControl.displayURL(webURL))
            {
                MDCMessageBox.showOpenURLError(applicationMainFrame, webURL);
            }
        }
        else
        {
            // tell user no URL assigned
            MDCMessageBox.showOpenURLError(applicationMainFrame);
        }
    }

    /**
     * Called to open the Open URL at specified Row.This currently uses the
     * BrowserControl from JavaWorld website that supports (windows/unix/mac)
     * and an error is displayed if this control fails to open resource
     * 
     * @see org.mdcog.client.desktop.utils.BrowserControl
     * @param row Zero based index of the row to open
     */
    public void openOpenURL(int row)
    {
        String openURL = ((ResourceItem) textFilteredResourceItems.get(row)).getOpenURL();
        // do we have a URL to open
        if (openURL.length() > 0)
        {
            // yes so open it
            if (!BrowserControl.displayURL(openURL))
            {
                MDCMessageBox.showOpenURLError(applicationMainFrame, openURL);
            }
        }
        else
        {
            // tell user no URL assigned
            MDCMessageBox.showOpenURLError(applicationMainFrame);
        }

    }

    /**
     * Called when the annotation button clicked at specified row. This grid
     * however does not show it so do nothing
     * 
     * @param row Zero based index of the row to open
     */
    public void openResourceItemAnnotation(int row)
    {
        // button does not exist on grid so will never be called
    }
}
