/*
 * MDC Desktop Client Created on 22-Apr-2005
 */
package org.mdcog.client.desktop.gui.controllers;

import java.awt.Cursor;
import java.io.FileNotFoundException;
import java.util.List;

import javax.swing.ButtonGroup;
import javax.swing.JFileChooser;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.filechooser.FileFilter;
import javax.swing.table.TableModel;

import org.mdcog.client.desktop.dataaccess.InvalidFileFormatException;
import org.mdcog.client.desktop.gui.CreateListDialog;
import org.mdcog.client.desktop.gui.EditAnnotationDialog;
import org.mdcog.client.desktop.gui.NewResourceItemDialog;
import org.mdcog.client.desktop.gui.OpenLocalURLDialog;
import org.mdcog.client.desktop.gui.error.MDCInternalException;
import org.mdcog.client.desktop.gui.error.MDCMessageBox;
import org.mdcog.client.desktop.gui.images.MDCIcons;
import org.mdcog.client.desktop.gui.models.MDCImportFileFilter;
import org.mdcog.client.desktop.gui.models.MDCLibraryFileFilter;
import org.mdcog.client.desktop.gui.models.ResourceItemTableFormat;
import org.mdcog.client.desktop.gui.models.ResourceListComparator;
import org.mdcog.client.desktop.gui.panels.SearchResultsGenericTabPanel;
import org.mdcog.client.desktop.model.ResourceItem;
import org.mdcog.client.desktop.model.ResourceList;
import org.mdcog.client.desktop.utils.BrowserControl;

import ca.odell.glazedlists.BasicEventList;
import ca.odell.glazedlists.EventList;
import ca.odell.glazedlists.SortedList;
import ca.odell.glazedlists.swing.EventTableModel;
import ca.odell.glazedlists.swing.TextFilterList;

/**
 * This controller controls the entire search tab panel, but ddelegates control
 * of the individual search result tabs to SearchResultsTabController. It makes
 * use of the Glazed Lists API to provide grid sorting and text filtering
 * 
 * @see <a href="http://publicobject.com/glazedlists">Glazed Lists HomePage </a>
 */
public class SearchTabController extends Controller implements ResourceItemButtonControllerInterface
{

    /**
     * Stores a reference to the maximum number of text charcaters shown on the
     * results tab
     */
    private static final int TAB_TEXT_LENGTH = 20;

    /**
     * Used to identify radio button selections between the dialog and
     * controller
     */
    public static final String SEARCH_KEYWORD = "SEARCH_KEYWORD";

    /**
     * Used to identify radio button selections between the dialog and
     * controller
     */
    public static final String SEARCH_AUTHOR = "SEARCH_AUTHOR";

    /**
     * Used to identify radio button selections between the dialog and
     * controller
     */
    public static final String SEARCH_ASSESSMENT = "SEARCH_ASSESSMENT";

    /**
     * Used to identify radio button selections between the dialog and
     * controller
     */
    public static final String SEARCH_BOOK = "SEARCH_BOOK";

    /**
     * Used to identify radio button selections between the dialog and
     * controller
     */
    public static final String SEARCH_JOURNAL = "SEARCH_JOURNAL";

    /**
     * Used to identify radio button selections between the dialog and
     * controller
     */
    public static final String SEARCH_IMAGE = "SEARCH_IMAGE";

    /**
     * Used to identify radio button selections between the dialog and
     * controller
     */
    public static final String SEARCH_VIDEO = "SEARCH_VIDEO";

    /**
     * Stores a reference to the singleton instance of this controller
     * 
     * @uml.property name="instance"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private static SearchTabController instance = new SearchTabController();

    /**
     * Stores a reference to the resource item table model for the results
     * resource list on the tab.
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private EventTableModel myListResourceTableModel = null;

    /**
     * Stores a reference to the my list resource items. This is in affect the
     * model for the tab as all data is temporary until moved to the main
     * library
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private EventList myListResourceEventList = null;

    /**
     * Stores a reference to the list of base my list resources in a sorted
     * order
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private SortedList myListSortedResources = null;

    /**
     * Stores a reference to the base my list resources in a text filtered order
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     */
    private TextFilterList myListTextFilteredResourceItems = null;

    /**
     * Stores a reference to the tab pane in the GUI so the controller can add
     * new tabs when search or imports complete
     */
    private JTabbedPane resultsTabPane = null;

    /**
     * Stores a reference to the search options radio button group that defines
     * mutual exclusivity between radio buttons
     */
    private ButtonGroup searchCriteriaButtons = null;

    /**
     * Stores a reference to the selector that will alter the selected tab
     * 
     * @uml.property name="tabSelector"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private TabSelectorInterface tabSelector = null;

    /**
     * contructor to the controller is private at this is a singleton
     */
    private SearchTabController()
    {
        // create the base list wheich will be empty to start
        myListResourceEventList = new BasicEventList();
        // create a sorted list of the added resources
        myListSortedResources = new SortedList(myListResourceEventList, new ResourceListComparator());
        // create the table format
        ResourceItemTableFormat tableFormat = new ResourceItemTableFormat(this);
        // create a filtered list of the sorted resources
        myListTextFilteredResourceItems = new TextFilterList(myListSortedResources, tableFormat);
        // store the filter text field on the table format so it can force a
        // refilter when columns removed or added
        tableFormat.setFilterTextField(myListTextFilteredResourceItems.getFilterEdit());
        // create the Resource Grid table model using the filtered list
        myListResourceTableModel = new EventTableModel(myListTextFilteredResourceItems, tableFormat);
    }

    /**
     * This Static method is used by the GUI to get a refrence to the controller
     * 
     * @return SearchTabController
     * @uml.property name="instance"
     */
    static public SearchTabController getInstance()
    {
        return instance;
    }

    /**
     * This method stores the tab selector into the controller should it need to
     * switch the tabs programatically
     * 
     * @param selector The selector to call to change tab
     * @uml.property name="tabSelector"
     */
    public void setTabSelector(TabSelectorInterface selector)
    {
        tabSelector = selector;
    }

    /**
     * Store the tab control for adding new panes
     * 
     * @param tabSet Refrence to the GUI tab control
     * @uml.property name="resultsTabPane"
     */
    public void setResultsTabPane(JTabbedPane tabSet)
    {
        resultsTabPane = tabSet;
    }

    /**
     * Store the button group for detecting sub search options
     * 
     * @param buttonGroup
     */
    public void setSearchOptionsButtonGroup(ButtonGroup buttonGroup)
    {
        searchCriteriaButtons = buttonGroup;
    }

    /**
     * Gets the resource table model resources grid
     * 
     * @return TableModel The table model for the table to use
     * @uml.property name="myListResourceTableModel"
     */
    public TableModel getMyListResourceTableModel()
    {
        return myListResourceTableModel;
    }

    /**
     * Returns the list of sorted items for the table comparator
     * 
     * @return the current list of sorted items
     */
    public SortedList getMyListSortedResourceItems()
    {
        return myListSortedResources;
    }

    /**
     * This returns the filter text control assciated with the resource items
     * grid so that it can be displayed on the screen.
     * 
     * @see <a
     *      href="http://publicobject.com/glazedlists/glazedlists-0.9.6/api/">Glazed
     *      Lists API </a>
     * @return JTextField the filter text control linked to resource item grid
     */
    public JTextField getMyListResourceItemsFilterTextControl()
    {
        return myListTextFilteredResourceItems.getFilterEdit();
    }

    /**
     * Called when the user requests a new search. It performs the search then
     * creates a new results controller and tab panel to display the results on
     * and adds it to the Tab control subsequently displaying it onto the screen
     * 
     * @param searchText The string to search under
     */
    public void newSearch(String searchText)
    {
        // did the user enter search text
        if (searchText.length() <= 0)
        {
            // display message to user to tell them they need to enter criteria
            MDCMessageBox.showEnterSearchCriteriaWarning(applicationMainFrame);
            return;
        }
        // perform the search. Bring back all resources for test
        List searchResults;

        Cursor standardCursor = applicationMainFrame.getCursor();
        
        try
        {
            
            applicationMainFrame.setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
            // perform search
            searchResults = getMDCLibraryModel().searchMDC(searchText);
            applicationMainFrame.setCursor(standardCursor);

            // make sure we had some results
            if (searchResults.size() > 0)
            {
                // create the controller
                SearchResultsTabController controller = new SearchResultsTabController(this, searchResults);
                // create the search tab panel
                SearchResultsGenericTabPanel tabPanel = new SearchResultsGenericTabPanel(controller);
                // if text is to long for tab trim and add ... to the end
                if (searchText.length() > TAB_TEXT_LENGTH)
                {
                    searchText = searchText.substring(0, TAB_TEXT_LENGTH) + "...";
                }
                resultsTabPane.addTab(searchText, MDCIcons.SEARCHSMALL_ICON, tabPanel);
                resultsTabPane.setSelectedIndex(resultsTabPane.getTabCount() - 1);
            }
            else
            {
                MDCMessageBox.showNoRecordsFound(searchText, applicationMainFrame);
            }
        }
        catch (MDCInternalException e)
        {
            applicationMainFrame.setCursor(standardCursor);
            MDCMessageBox.showInternalError(e, applicationMainFrame);
        }

    }

    /**
     * This method iterates the array of resource items and adds them to the
     * myListResourceEventList model as long as they do not already exist
     * 
     * @param items An array or ResourceItems to add
     */
    public void addResourceItemsToMyList(ResourceItem[] items)
    {
        for (int i = 0; i < items.length; i++)
        {
            // only add item if it does not already exist in list
            if (!myListResourceEventList.contains(items[i]))
            {
                // addding to list will auto refresh screen
                myListResourceEventList.add(items[i]);
            }
        }
    }

    /**
     * This method saves the specified resource items rows in MyList grid into
     * the main MDCLibraryModel where it will be permently persisted. Upto this
     * point all data on the screen is transient and will be deleted on
     * application termination. This method delegates the actual adding of the
     * resource items to the LibraryTabController. The saved items are deleted
     * from the myListResourceEventList model once successfully moved. If the
     * user has not selected any rows then this method will move them all by
     * default.
     * 
     * @see org.mdcog.client.desktop.gui.controllers.LibraryTabController#addNewResourceListWithItemsToMyLibary(ResourceItem[])
     * @param rows array of zero based indexs of selected rows
     */
    public void saveToMyLibrary(int[] rows)
    {
        ResourceItem[] resourceItems = null;

        // If the user has selected no rows at all then move everything
        if (rows.length == 0)
        {
            // create array with enough room to store results
            resourceItems = new ResourceItem[myListTextFilteredResourceItems.size()];
            // populate resource items with all rows
            resourceItems = (ResourceItem[]) myListTextFilteredResourceItems.toArray(resourceItems);
        }
        else
        {
            // create an array to hold resource items to copy
            resourceItems = getMyListResourceItemsForRows(rows);
        }

        // ask the myLibrary tab controller to add the resources to MyList grid
        LibraryTabController.getInstance().addNewResourceListWithItemsToMyLibary(resourceItems);

        //loop round and get the items to delete. Could call delete method
        // but as we already know list easier to delete here
        for (int i = 0; i < resourceItems.length; i++)
        {
            myListTextFilteredResourceItems.remove(resourceItems[i]);
        }

        // switch focus to the other library tab
        tabSelector.selectLibraryTab();

    }

    /**
     * This method deletes the Resource Items for the specifiedrows from the
     * myListResourceEventList model
     * 
     * @param rows An array of zero based indexs of the selected row
     */
    public void deleteResourceItemsFromMyList(int[] rows)
    {
        ResourceItem[] resourceItems = getMyListResourceItemsForRows(rows);
        //loop round and get the items to delete
        for (int i = 0; i < rows.length; i++)
        {
            myListTextFilteredResourceItems.remove(resourceItems[i]);
        }
    }

    /**
     * Extracts the actual resource items for the specifed rows and returns them
     * in an array
     * 
     * @param rows An array of zero based indexs of the selected row
     * @return An array of resource items that correspond to rows
     */
    public ResourceItem[] getMyListResourceItemsForRows(int[] rows)
    {
        // create an array to hold resource items
        ResourceItem[] resourceItems = new ResourceItem[rows.length];

        // loop round and get the items to delete
        for (int i = 0; i < rows.length; i++)
        {
            // copy the references into array from text filtered list as this is
            // the list the indexs match against
            resourceItems[i] = (ResourceItem) myListTextFilteredResourceItems.get(rows[i]);
        }
        return resourceItems;
    }

    /**
     * Closes the specified Tab and cleans up all resources
     * 
     * @param tabToClose The index of the tab to close
     */
    public void closeSearchResultsTab(int tabToClose)
    {
        // first get the componet we are going to close
        SearchResultsGenericTabPanel gridToClose = (SearchResultsGenericTabPanel) resultsTabPane.getComponentAt(tabToClose);
        // now remove from tabbed panel
        resultsTabPane.remove(tabToClose);
        // now tell the Grid Panel to cleanup data
        gridToClose.closeAndCleanUp();
    }

    /**
     * Closes all the search result tabs
     */
    public void closeAllSearchResultsTabs()
    {
        // loop round closing each tab starting at the end so we do not get
        // index problems
        for (int i = resultsTabPane.getTabCount(); i > 0; i--)
        {
            // close tab using close tab method so we ensure we always clean up
            // in same way
            // need to subtrat 1 as tab indexs are 0 based
            closeSearchResultsTab(i - 1);
        }
    }

    /**
     * Opens the create list dialog with a copy of the currently selected
     * resource list data in MyList grid. A message will be displayed if this is
     * attempted on a list with no items. <br>
     * <br>
     * This method delegates all ResourceList and list related ResourceItem
     * model processing to the MyLibaryResourceListTableModel as this model
     * controls the handling of the resource lists screen grid data and is
     * better suited to being the single point of interaction with the actual
     * main MDCLibraryModel.
     * 
     * @see org.mdcog.client.desktop.gui.models.ResourceListTableModel
     */
    public void createList()
    {
        // before we do anything check we have resource items
        if (myListResourceEventList.size() > 0)
        {
            ResourceItem clonedItem = null;
            // create a new library resource list entry with a dummy name
            ResourceList entry = new ResourceList("MyList Entry");
            // loop round adding all entries
            for (int i = 0; i < myListResourceEventList.size(); i++)
            {
                // clone the entry so any changes are not reflected into my list
                clonedItem = (ResourceItem) ((ResourceItem) myListResourceEventList.get(i)).clone();
                entry.addNewResourceItem(clonedItem);
            }
            // create the popup dialogue
            CreateListDialog dialog = new CreateListDialog(entry, applicationMainFrame);
            // show the dilaogue.
            dialog.show();
        }
        else
        {
            MDCMessageBox.showCreateListWarning(applicationMainFrame);
        }
    }

    /**
     * Called to import a list from the file menu. This ensures the correct tab
     * is selected then calls normal import list functionality
     */
    public void fileMenuImport()
    {
        tabSelector.selectSearchTab();
        importResults();
    }

    /**
     * This method imports either downloaded search results or another users
     * master library. It performs the import then creates a new results
     * controller and tab panel to display the results on and adds it to the Tab
     * control subsequently displaying it onto the screen. Errors are shown if
     * the file is not found or contains an invlid format.
     */
    public void importResults()
    {
        // popup dialogue box to get the file name to open
        JFileChooser fileChooser = new JFileChooser(getMDCLibraryModel().getCurrentLibraryFileName());
        fileChooser.setMultiSelectionEnabled(false);
        // set the selection options
        fileChooser.setFileFilter((FileFilter) new MDCImportFileFilter());
        // show the dialog
        int option = fileChooser.showOpenDialog(applicationMainFrame);
        // did we select a file to load
        if (option == JFileChooser.APPROVE_OPTION)
        {
            String filename = fileChooser.getSelectedFile().getAbsolutePath();
            try
            {
                List searchResults = null;
                // if extension is library file load it esle assume we are xml
                if (MDCLibraryFileFilter.isLibraryFile(fileChooser.getSelectedFile()))
                {
                    // ask the model to load the library file
                    searchResults = getMDCLibraryModel().extractAllResourcesFromLibraryFile(filename);
                }
                else
                {
                    searchResults = getMDCLibraryModel().extractAllResourcesFromImportFile(filename);
                }
                // create the controller
                SearchResultsTabController controller = new SearchResultsTabController(this, searchResults);
                // create the search tab panel with filename as tab header
                SearchResultsGenericTabPanel tabPanel = new SearchResultsGenericTabPanel(controller);
                String fileName = fileChooser.getSelectedFile().getName();
                //if text is to long for tab trim and add ... to the end
                if (fileName.length() > TAB_TEXT_LENGTH)
                {
                    fileName = fileName.substring(0, TAB_TEXT_LENGTH) + "...";
                }
                resultsTabPane.addTab(fileName, MDCIcons.SEARCHSMALL_ICON, tabPanel);
                resultsTabPane.setSelectedIndex(resultsTabPane.getTabCount() - 1);
            }
            catch (FileNotFoundException e)
            {
                MDCMessageBox.showFileNotFound(filename, applicationMainFrame);
            }
            catch (InvalidFileFormatException e)
            {
                // loading the MDC library file found invlaid format
                MDCMessageBox.showInvalidFileFormat(filename, applicationMainFrame);
            }
        }
    }

    /**
     * This method starts the NewResourceItemDialog to allow the user to add new
     * ResourceItems to the myList resource list
     */
    public void createResourceItems()
    {
        // create the pop up
        NewResourceItemDialog dialog = new NewResourceItemDialog(applicationMainFrame);
        // show the dialog
        dialog.show();

        // ask the model to add the resources to selected row and all resources
        myListTextFilteredResourceItems.addAll(dialog.getCreatedResources());
        // notify the listsners that the model has changed
        notifyListenersOfModelChange();
    }

    /**
     * This method is called to open the local URL for the specifed resource
     * item row.
     * 
     * @param row Zero based index of the row to open
     */
    public void openLocalURL(int row)
    {
        // create the open URL dialog with the resource item model at specified
        // row
        OpenLocalURLDialog openURLDialog = new OpenLocalURLDialog(applicationMainFrame,
                (ResourceItem) myListTextFilteredResourceItems.get(row));
        openURLDialog.show();
    }

    /**
     * Called to open the Web URL at specified Row.This currently uses the
     * BrowserControl from JavaWorld website that supports (windows/unix/mac)
     * and an error is displayed if this control fails to open resource
     * 
     * @see org.mdcog.client.desktop.utils.BrowserControl
     * @param row Zero based index of the row to open
     */
    public void openWebURL(int row)
    {
        String webURL = ((ResourceItem) myListTextFilteredResourceItems.get(row)).getWebURL();
        // do we have a URL to open
        if (webURL.length() > 0)
        {
            // yes so open it
            if (!BrowserControl.displayURL(webURL))
            {
                MDCMessageBox.showOpenURLError(applicationMainFrame, webURL);
            }
        }
        else
        {
            // tell user no URL assigned
            MDCMessageBox.showOpenURLError(applicationMainFrame);
        }
    }

    /**
     * Called to open the Open URL at specified Row.This currently uses the
     * BrowserControl from JavaWorld website that supports (windows/unix/mac)
     * and an error is displayed if this control fails to open resource
     * 
     * @see org.mdcog.client.desktop.utils.BrowserControl
     * @param row Zero based index of the row to open
     */
    public void openOpenURL(int row)
    {
        String openURL = ((ResourceItem) myListTextFilteredResourceItems.get(row)).getOpenURL();
        // do we have a URL to open
        if (openURL.length() > 0)
        {
            // yes so open it
            if (!BrowserControl.displayURL(openURL))
            {
                MDCMessageBox.showOpenURLError(applicationMainFrame, openURL);
            }
        }
        else
        {
            // tell user no URL assigned
            MDCMessageBox.showOpenURLError(applicationMainFrame);
        }

    }

    /**
     * Called when the annotation button clicked at specified row
     * 
     * @param row Zero based index of the row to open
     */
    public void openResourceItemAnnotation(int row)
    {
        // get the annptation for the specified row
        String annotation = ((ResourceItem) myListTextFilteredResourceItems.get(row)).getAnnotation();
        // create the edit annotation dialog
        EditAnnotationDialog dialog = new EditAnnotationDialog(annotation, applicationMainFrame);
        dialog.show();
        // if the annotation was updated store it back onto resource item
        if (dialog.annotationUpdated())
        {
            // store the new annotation back on item
            ((ResourceItem) myListTextFilteredResourceItems.get(row)).setAnnotation(dialog.getAnnotation());
            // notify model change
            notifyListenersOfModelChange();
        }
    }
}
