/*
 * MDC Desktop Client Created on 26-Apr-2005
 */
package org.mdcog.client.desktop.gui.models;

import java.util.Comparator;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.TreeMap;

import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

import org.mdcog.client.desktop.gui.controllers.ResourceItemButtonControllerInterface;
import org.mdcog.client.desktop.gui.error.MDCInternalException;
import org.mdcog.client.desktop.gui.images.MDCIcons;
import org.mdcog.client.desktop.gui.menus.SelectResourceItemColumnsPopUpMenu;
import org.mdcog.client.desktop.gui.table.renderers.ResourceItemURLRenderer;
import org.mdcog.client.desktop.model.ResourceItem;

import ca.odell.glazedlists.TextFilterator;
import ca.odell.glazedlists.gui.AdvancedTableFormat;
import ca.odell.glazedlists.impl.sort.ComparableComparator;

/**
 * Defines the Table format for all the main resource grids. Can be extended to
 * alter tables slightly. This class is heavly linked to the ResourceItem class
 * that it displays data about.
 * 
 * @see <a href="http://publicobject.com/glazedlists">Glazed Lists HomePage </a>
 */
public class ResourceItemTableFormat implements AdvancedTableFormat, ColumnDisplayInterface, TextFilterator
{
    /**
     * Stores a reference to the table row height 
     */
    public final static int ROW_HEIGHT = 22;
    /**
     * Stores a reference to the filter text field so we can force a refresh
     * when column removed or added
     */
    private JTextField filterTextField = null;

    /**
     * Stores a reference to the operating table when controlling display
     * columns
     */
    private JTable operatingTable = null;

    /**
     * Stores a reference to the operating column that was last clicked
     */
    private int operatingColumn = 0;

    /**
     * Stores a reference to the comparer that is used to sort columns
     */
    protected ComparableComparator comparableComparator = null;

    /**
     * Stores a reference to the ler that will handle the URL button clicks
     * 
     * @uml.property name="controller"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    protected ResourceItemButtonControllerInterface controller = null;

    /**
     * Stores a reference to the number of displayable columns
     */
    protected final static int NUMOFCOLUMNS = 11;

    /**
     * Stores a reference to the columns that are displayed and can be toggled
     * on and off
     */
    protected Map columnToggleDisplayMap = null;

    /**
     * Contructor
     * 
     * @param controller The controller to process URL button clicks
     */
    public ResourceItemTableFormat(ResourceItemButtonControllerInterface controller)
    {
        this.controller = controller;
        comparableComparator = new ComparableComparator();
        columnToggleDisplayMap = new TreeMap(new TableColumnComparator());
    }

    /**
     * Returns the number of columns displayed
     * 
     * @return the number of columns to display
     */
    public int getColumnCount()
    {
        return NUMOFCOLUMNS;
    }

    /**
     * sets the filter text field relative to the table so when columns are
     * added or removed we can automate a refresh
     * 
     * @param field The text filed control
     */
    public void setFilterTextField(JTextField field)
    {
        filterTextField = field;
    }

    /**
     * Returns the name for the column 
     * @param columnIndex the column that requires it header title 
     * @return The header
     */
    public String getColumnName(int columnIndex)
    {
        switch (columnIndex)
        {
        case 0:
            return "Author";
        case 1:
            return "Title";
        case 2:
            return "Publisher";
        case 3:
            return "Type";
        case 4:
            return "ISBN";
        case 5:
            return "ISSN";
        case 6:
            return "Abstract";
        case 7:
            return "Annotation";
        case 8:
            return "Local URL";
        case 9:
            return "Web URL";
        case 10:
            return "Open URL";

        }
        return "Undefined";
    }

    /**
     * Extracts from the baseObject the data for the specific column
     * 
     * @param baseObject The base Object (ResourceItem)
     * @param columnIndex the column that requires its value
     * @return The extracted data for the column
     */
    public Object getColumnValue(Object baseObject, int columnIndex)
    {
        // make sure not null object
        if (baseObject == null)
        {
            return null;
        }
        // cast object to correct type
        ResourceItem item = (ResourceItem) baseObject;
        // extract correct data
        switch (columnIndex)
        {
        case 0:
            return item.getAuthor();
        case 1:
            return item.getTitle();
        case 2:
            return item.getPublisher();
        case 3:
            return item.getType();
        case 4:
            return item.getISBN();
        case 5:
            return item.getISSN();
        case 6:
            return item.getAbstractStr();
        case 7:
            return item.getAnnotation();
        case 8:
            return item.getLocalURL();
        case 9:
            return item.getWebURL();
        case 10:
            return item.getOpenURL();
        }
        return null;
    }

    /**
     * Returns the class for the column inorder to implement custom renderes
     * 
     * @param columnIndex the column that requires it column class type
     * @return All columns are of type String
     */
    public Class getColumnClass(int columnIndex)
    {
        return String.class;
    }

    /**
     * Returns the comparator to use for the column
     * 
     * @param columnIndex the column that requires its comparator
     * @return The standard String comparator or null when button columns
     */
    public Comparator getColumnComparator(int columnIndex)
    {
        // *** NOTE IF YOU CHANGE THE FORMAT HERE NEED TO ALTER
        // SearchResultResourceItemTableFormat AS WELL ***
        switch (columnIndex)
        {
        case 0:
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
        case 6:
            return comparableComparator;
        case 7:
        case 8:
        case 9:
        case 10:
            return null;
        }
        return null;
    }

    /**
     * Set up column specific attributes when we start
     * 
     * @param column The column class for the index
     * @param columnIndex the column that requires setting up
     */
    public void setUpColumn(TableColumn column, int columnIndex)
    {
        // add to diaplayable map
        addColumnToToggleDisplayMap(column, columnIndex);
        // set up renderer
        TableCellRenderer renderer = getColumnRender(columnIndex);
        if (renderer != null)
        {
            column.setCellRenderer(renderer);
        }
        // set up widths
        setUpColumnStartUpWidths(column, columnIndex);
    }

    /**
     * Sets up the initial column widths to fix button sizes and stop them from
     * being resized
     * 
     * @param column The column class for the index
     * @param columnIndex the column that requires setting up
     */
    protected void setUpColumnStartUpWidths(TableColumn column, int columnIndex)
    {
        switch (columnIndex)
        {
        case 0:
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
        case 6:
            break;
        case 7: 
            column.setMaxWidth(77);
            column.setMinWidth(77);
            column.setWidth(77);
            column.setResizable(false);
            break;
        case 8:
        case 9:
        case 10:
            column.setMaxWidth(75);
            column.setMinWidth(75);
            column.setWidth(75);
            column.setResizable(false);

        }
    }

    /**
     * Returns the special column render for given column numbers null if no
     * specific one applies. This sets up the listeners and controller calls for
     * the button renderers to process mouse clicks
     * 
     * @param columnIndex the column that requires setting up
     * @return The renderer for the column
     */
    protected TableCellRenderer getColumnRender(int columnIndex)
    {
        // *** NOTE IF YOU CHANGE THE FORMAT HERE NEED TO ALTER
        // SearchResultResourceItemTableFormat AS WELL ***
        ResourceItemURLRenderer renderer = null;
        switch (columnIndex)
        {
        case 0:
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
        case 6:
            return null;
        case 7:
            // create the URL renderer and add listner for when its clicked
            renderer = new ResourceItemURLRenderer("...", MDCIcons.ANNOTATION_ICON);
            renderer.addURLListener(new ResourceItemButtonListener()
            {

                public void buttonClicked(int row)
                {
                    controller.openResourceItemAnnotation(row);
                }
            });
            return renderer;
        case 8:
            // create the URL renderer and add listner for when its clicked
            renderer = new ResourceItemURLRenderer("...", MDCIcons.OPENSMALL_ICON);
            renderer.addURLListener(new ResourceItemButtonListener()
            {

                public void buttonClicked(int row)
                {
                    controller.openLocalURL(row);
                }
            });
            return renderer;
        case 9:
            // create the URL renderer and add listner for when its clicked
            renderer = new ResourceItemURLRenderer("...", MDCIcons.WORLD_ICON);
            renderer.addURLListener(new ResourceItemButtonListener()
            {

                public void buttonClicked(int row)
                {
                    controller.openWebURL(row);
                }
            });
            return renderer;
        case 10:
            // create the URL renderer and add listner for when its clicked
            renderer = new ResourceItemURLRenderer("...", MDCIcons.WORLD_ICON);
            renderer.addURLListener(new ResourceItemButtonListener()
            {

                public void buttonClicked(int row)
                {
                    controller.openOpenURL(row);
                }
            });
            return renderer;
        }
        return null;
    }

    /**
     * Adds the column to the display map if it can be toggled on and off
     * 
     * @param column The column class for the index
     * @param columnIndex the column that requires setting up
     */
    protected void addColumnToToggleDisplayMap(TableColumn column, int columnIndex)
    {
        // only the first six columns can be toggled on and off
        switch (columnIndex)
        {
        case 0:
        case 1:
        case 2:
        case 3:
        case 4:
        case 5:
        case 6:
            // add to map and flag as currently displayed
            columnToggleDisplayMap.put(column, new Boolean(true));
        }
    }

    /**
     * Shows a popup of all the columns currently selected
     * 
     * @param table the table to show pop-up againnst
     * @param columnClicked The column index clicked
     * @param x X Position for pop up
     * @param y Y Position for pop up
     * @throws MDCInternalException
     */
    public void showSelectedColumnsPopUp(JTable table, int columnClicked, int x, int y) throws MDCInternalException
    {
        // store the current operating table for the call back processing
        this.operatingTable = table;
        this.operatingColumn = columnClicked;
        // create and display the pop up
        SelectResourceItemColumnsPopUpMenu popup = new SelectResourceItemColumnsPopUpMenu(this, columnToggleDisplayMap);
        // make sure we show popup relative to header
        popup.show(table.getTableHeader(), x, y);
    }

    /**
     * Call back from menu to show or hide the specified column
     * 
     * @param column The column to show or hide
     * @param display true if we are to display this column false if we are
     *        hiding
     */
    public void showColumn(TableColumn column, boolean display)
    {
        // are we displaying
        if (display)
        {
            // add the column to the table
            operatingTable.addColumn(column);
            // move column to before the last operating column
            operatingTable.moveColumn(getColumnCount() - 1, operatingColumn);
            // set the column as displayed in display map
            columnToggleDisplayMap.put(column, new Boolean(true));
        }
        else
        {
            // remove the column from the table
            operatingTable.removeColumn(column);
            // set the column as hidden in display map
            columnToggleDisplayMap.put(column, new Boolean(false));
        }

        // as a column has been removed or added we want to force a filter
        // refresh to do this the filter text filed must be set
        if (filterTextField != null)
        {
            // unfortuantly glazed lists is clever and detects no change in the
            // filter text so it does not reperform the filter opperation. To do
            // this we need to basically blank the filed and reset the text.
            // This is obviously not the most performant and will need to be
            // tested with a long list. However it is the only solution
            // available at present
            String s = filterTextField.getText();
            filterTextField.setText("");
            filterTextField.setText(s);
        }

    }

    /**
     * Sets up the columns that are currently text filterable
     * 
     * @param baseList the base list of items
     * @param element Each individual item
     */
    public void getFilterStrings(List baseList, Object element)
    {
        // get an iterator round the column display map keys
        Iterator iter = columnToggleDisplayMap.keySet().iterator();

        TableColumn column = null;

        while (iter.hasNext())
        {
            // get the next key which is the table column
            column = (TableColumn) iter.next();
            // we only add the columns data to the current base list of
            // filterable strings if it is displayed
            if (((Boolean) columnToggleDisplayMap.get(column)).booleanValue())
            {
                // add the column value to the base list
                baseList.add(getColumnValue(element, column.getModelIndex()));
            }
        }
    }

}
