/*
 * MDC Desktop Client Created on 25-Apr-2005
 */
package org.mdcog.client.desktop.gui.table.editors;

import java.awt.Component;
import java.awt.event.FocusEvent;
import java.awt.event.MouseEvent;
import java.util.EventObject;
import java.util.Vector;

import javax.swing.JTable;
import javax.swing.JTextField;
import javax.swing.event.CellEditorListener;
import javax.swing.event.ChangeEvent;
import javax.swing.table.TableCellEditor;

import org.mdcog.client.desktop.model.ResourceList;

/**
 * This class is called when the user clicks or selects to rename a resource
 * list. It controls the editing of the value
 */
public class ResourceListEntryEditor extends JTextField implements TableCellEditor
{

    /**
     * Stores a reference to the object listening for changes
     */
    protected Vector listeners;

    /**
     * Stores a reference to the whether this is the first time focus has been
     * gained to auto select text
     */
    protected boolean firstfocus = false;

    /**
     * Stores a reference to the entry being modified
     * 
     * @uml.property name="entry"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    protected ResourceList entry;

    /**
     * Stores a reference to the number of mouse clicks required to start
     * editing mode
     */
    private static final int CLICKSTOSTART = 2;

    /**
     * Constructor
     */
    public ResourceListEntryEditor()
    {
        listeners = new Vector();
    }

    /**
     * This method is called when the editor processes a focus event. It checks
     * to see if focus is gained for the first time and then selects all the
     * text to be edited
     * 
     * @param fe The focus event
     */
    protected void processFocusEvent(FocusEvent fe)
    {
        if (fe.getID() == FocusEvent.FOCUS_GAINED)
        {
            // only do select all if first focus flag is true
            if (firstfocus)
            {
                selectAll();
                firstfocus = false;
            }
        }
        super.processFocusEvent(fe);
    }

    /**
     * This is called to get the component editor
     * 
     * @param table The table the editor is against
     * @param value The data to be edited
     * @param isSelected indicates if currently selected
     * @param row The cell row
     * @param column the cell column
     * @return The component that does the editing
     */
    public Component getTableCellEditorComponent(JTable table, Object value, boolean isSelected, int row, int column)
    {
        entry = (ResourceList) value;
        // store the entry name into text field
        setText(entry.getResourceListName());
        // set first focus flag to true so that all text is selected
        firstfocus = true;

        return this;
    }

    /**
     * Called when the editing is cancelled. The new value is not stored back.
     * This method informs all listeners that editing has completed
     */
    public void cancelCellEditing()
    {
        // signal canceled editing to any listeners
        ChangeEvent change = new ChangeEvent(this);
        for (int looper = listeners.size() - 1; looper >= 0; looper--)
        {
            ((CellEditorListener) listeners.elementAt(looper)).editingCanceled(change);
        }
    }

    /**
     * Called when the editing is stopped.This method stores the value and
     * informs all listeners that editing has completed
     * 
     * @return Whether cell editing stopped correctly
     */
    public boolean stopCellEditing()
    {
        // only store the new name if it is not empty
        String newName = getText();
        if (newName.length() > 0)
        {
            entry.setResourceListName(newName);
        }
        // signal stopped editing to any listeners
        ChangeEvent change = new ChangeEvent(this);
        for (int looper = listeners.size() - 1; looper >= 0; looper--)
        {
            ((CellEditorListener) listeners.elementAt(looper)).editingStopped(change);
        }
        return true;
    }

    /**
     * Gets the value of the cell beoing edited
     * 
     * @return The ResourceList being edited
     */
    public Object getCellEditorValue()
    {
        return entry;
    }

    /**
     * Return a boolean if the cell allows editin to start. This caters for the
     * double click to start editing
     * 
     * @param event The cell event
     * @return true if editing can start
     */
    public boolean isCellEditable(EventObject event)
    {
        //must have clicked mouse CLICKSTOSTART times to edit
        if (event instanceof MouseEvent)
        {
            return ((MouseEvent) event).getClickCount() >= CLICKSTOSTART;
        }
        return true;

    }

    /** 
     * Return if true that the cell should auto select on editing
     * @param event The cell event
     * @return true
     */
    public boolean shouldSelectCell(EventObject event)
    {
        return true;
    }

    /** 
     * Add a cell listener
     * @param cell The CellEditorListener that wants to register
     */
    public void addCellEditorListener(CellEditorListener cell)
    {
        listeners.addElement(cell);
    }

    /** 
     * Remove a cell listener
     * @param cell The CellEditorListener that wants to un-register
     */
    public void removeCellEditorListener(CellEditorListener cell)
    {
        listeners.removeElement(cell);
    }

}
