/*
 * MDC Desktop Client Created on 22-Apr-2005
 */
package org.mdcog.client.desktop.model;

import java.io.FileNotFoundException;
import java.util.Collection;
import java.util.List;
import java.util.Vector;

import org.mdcog.client.desktop.dataaccess.InvalidFileFormatException;
import org.mdcog.client.desktop.dataaccess.LibraryDAO;
import org.mdcog.client.desktop.gui.error.MDCInternalException;
import org.mdcog.client.styles.MDCStyle;

/**
 * This is the master LibraryModel where all the business and data for the
 * client is retrieved and managed. It is also a singleton as it maintains data
 * during the life of the application
 * 
 * @uml.dependency supplier="org.mdcog.client.desktop.model.ResourceList" stereotypes="Basic::Responsibility"
 * @uml.dependency supplier="org.mdcog.client.styles.MDCStyle" stereotypes="Basic::Responsibility"
 */

public class MDCLibraryModel extends MDCModel
{

    /**
     * Stores a reference to the resource location for the styles configuration
     * XML file
     */
    public static final String STYLES_XML = "org/mdcog/client/styles/styles.xml";

    /**
     * Stores a reference to the library field type extension
     */
    public static final String LIBRARY_FILE_EXTENSION = "rly";

    /**
     * Stores a reference to the export (xml) type extension
     */
    public static final String EXPORT_FILE_EXTENSION = "xml";

    /**
     * Stores a reference to the create list html (html) type extension
     */
    public static final String CREATE_LIST_HTML_FILE_EXTENSION = "html";

    /**
     * Stores a reference to the create list pdf (pdf) type extension
     */
    public static final String CREATE_LIST_PDF_FILE_EXTENSION = "pdf";

    /**
     * Stores a reference to the the singleton instance
     * 
     * @uml.property name="instance"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private static MDCLibraryModel instance = new MDCLibraryModel();


    /**
     * Stores a reference to the names of all systems resource lists The first
     * element always represents the master list (all resources) Subsequent
     * entries are users personal lists.
     */
    private Vector myLibraryListsModel = null;

    /**
     * Stores a reference to the currently loaded filename
     */
    private String currentLibraryFileName = "";

    /**
     * Stores a reference to the data access layer component that loads and
     * stores data
     * 
     * @uml.property name="libraryDAO"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private LibraryDAO libraryDAO = null;


    /**
     * contructor to the MDCLibraryModel is private at this is a singleton
     */
    private MDCLibraryModel()
    {
        createBasicListModel();
        libraryDAO = new LibraryDAO();
    }

    /**
     * This Static method is used to get a refrence to the MDCLibraryModel
     * 
     * @return The instance of MDCLibraryModel
     * 
     * @uml.property name="instance"
     */
    static public MDCLibraryModel getInstance() {
        return instance;
    }


    /**
     * Create the basic list model for use. This currently creates dummy data
     */
    private void createBasicListModel()
    {
        // create a new vector and add in the All resources entry
        myLibraryListsModel = new Vector();
        // first item index 0 is always the master list containing main objects
        // create a blank all resources list
        myLibraryListsModel.add(ResourceList.createBlankAllResources());
        // set clean as creating dummy data will set data dirty
        setClean();
    }

    /**
     * Sets the model clean and cleans all LibraryResourceEntries
     * 
     * @return true if this model or any library resource entry is dirty and
     *         needs saving
     */
    public boolean isDirty()
    {
        // is this model dirty
        if (super.isDirty())
        {
            // no need to check anymore we are dirty
            return true;
        }

        // still clean so check
        for (int i = 0; i < myLibraryListsModel.size(); i++)
        {
            // check each resource list item to see if it is dirty
            if (((ResourceList) myLibraryListsModel.get(i)).isDirty())
            {
                //  no need to check anymore we are dirty
                return true;
            }
        }
        return false;
    }

    /**
     * Sets the model clean and cleans all LibraryResourceEntries
     */
    public void setClean()
    {
        super.setClean();
        // clean all ResourceList's
        for (int i = 0; i < myLibraryListsModel.size(); i++)
        {
            ((ResourceList) myLibraryListsModel.get(i)).setClean();
        }
    }

    /**
     * Retrieves the currently loaded files name
     * 
     * @return Returns the currentLibraryFileName.
     * 
     * @uml.property name="currentLibraryFileName"
     */
    public String getCurrentLibraryFileName() {
        return currentLibraryFileName;
    }

    /**
     * Sets the currently loaded filename
     * 
     * @param currentLibraryFileName The currentLibraryFileName to set.
     * 
     * @uml.property name="currentLibraryFileName"
     */
    public void setCurrentLibraryFileName(String currentLibraryFileName) {
        this.currentLibraryFileName = currentLibraryFileName;
    }

    /**
     * Removes the current library and creates an entire new one
     */
    public void newLibrary()
    {
        // clear current library
        myLibraryListsModel.clear();
        // create a new basic list model
        myLibraryListsModel.add(ResourceList.createBlankAllResources());
        // blank out the current file name
        currentLibraryFileName = "";
        // set the model clean again
        setClean();
    }

    /**
     * Removes the current library and opens the specified one
     * 
     * @param filename The filename to load
     * @throws FileNotFoundException
     * @throws InvalidFileFormatException
     */
    public void openLibrary(String filename) throws FileNotFoundException, InvalidFileFormatException
    {
        // make sure we can get the data before we clear anything
        Collection newLibrary = libraryDAO.loadLibrary(filename);
        // clear current library
        myLibraryListsModel.clear();
        // get the data access layer to open and return new collection of
        // library entries
        myLibraryListsModel.addAll(newLibrary);
        // blank out the current file name
        currentLibraryFileName = filename;
        // set the model clean again
        setClean();
    }

    /**
     * Saves the current library
     * 
     * @param filename The filename to load or NULL if to use currently loaded
     *        filename
     * @throws FileNotFoundException
     * @throws MDCInternalException
     */
    public void saveLibrary(String filename) throws FileNotFoundException, MDCInternalException
    {
        // did we specify a new name
        if (filename != null)
        {
            // save the new name to save under
            currentLibraryFileName = filename;
        }
        // get the data access layer to osave the library
        libraryDAO.saveLibrary(currentLibraryFileName, myLibraryListsModel);
        // set the model clean again
        setClean();
    }

    /**
     * Opens the specified library file and parses out all the resource Items it
     * contains
     * 
     * @param filename The file to open
     * @return The list or retrieved resource items
     * @throws FileNotFoundException
     * @throws InvalidFileFormatException
     */
    public List extractAllResourcesFromLibraryFile(String filename) throws FileNotFoundException, InvalidFileFormatException
    {
        // load the library
        List newLibrary = libraryDAO.loadLibrary(filename);
        // extract the master list
        ResourceList entry = (ResourceList) newLibrary.get(0);
        // return the master lists resources
        return entry.getResourceItems();
    }

    /**
     * Opens the specified import file and parses out all the resource Items it
     * contains
     * 
     * @param filename The file to open
     * @return The list or retrieved resource items
     * @throws FileNotFoundException
     * @throws InvalidFileFormatException
     */
    public List extractAllResourcesFromImportFile(String filename) throws FileNotFoundException, InvalidFileFormatException
    {
        // load the library
        return libraryDAO.importResources(filename);
    }

    /**
     * Returns all the resource lists
     * 
     * @return List or ResourceList elements
     */
    public List getAllResourceLists()
    {
        return (List) myLibraryListsModel;
    }

    /**
     * Gets the ResourceList specified by the index
     * 
     * @param index the index to retrieve
     * @return The ResourceList
     */
    private ResourceList getResourceListFromMyLibrary(int index)
    {
        return (ResourceList) myLibraryListsModel.get(index);
    }

    /**
     * This method adds a new resource list to the users library
     * 
     * @param items The resource items to add whilst creating. Null if none to
     *        be entered
     * @return The index it was inserted at in the library
     */
    public int addNewResourceListToMyLibrary(ResourceItem[] items)
    {
        // create a new entry
        ResourceList entry = new ResourceList("New Personal Folder");
        // if items specifed then add them now
        if (items != null)
        {
            //Get the all resources entry as all users list resources by
            //definition must be in here as well
            ResourceList allResourcesEntry = getResourceListFromMyLibrary(0);
            // loop round adding resources
            for (int i = 0; i < items.length; i++)
            {
                // need to add to both all resources and new entry
                allResourcesEntry.addNewResourceItem(items[i]);
                entry.addNewResourceItem(items[i]);
            }
        }
        // add the new folder to the list of library folders
        myLibraryListsModel.add(entry);
        // set this model dirty
        setDirty();
        // return the number of lists in library as this new one will now be the
        // last
        return myLibraryListsModel.size();
    }

    /**
     * This method deletes a resource list from the users library You can not
     * delete all resources entry as this is fixed
     * 
     * @param entry The resource list to delete
     */
    public void deleteResourceListFromMyLibrary(ResourceList entry)
    {
        // Can only remove if item exists and index is greater than 0
        if (myLibraryListsModel.indexOf(entry) > 0)
        {
            myLibraryListsModel.remove(entry);
            // set this model dirty
            setDirty();
        }
    }

    /**
     * Return the number or resources lists in the library
     * 
     * @return count of the resource lists in the model
     */
    public int getNumberOfResourceListsInMyLibrary()
    {
        return myLibraryListsModel.size();
    }

    /**
     * Add the resource item to the specified resource list
     * 
     * @param resourceListIndex The resource list index to add to
     * @param resourceItem The resource item to add
     */
    private void deleteResourceItemFromResourceListInMyLibrary(int resourceListIndex, ResourceItem resourceItem)
    {
        // add the new entry to the correct resource list
        ((ResourceList) myLibraryListsModel.elementAt(resourceListIndex)).deleteResourceItem(resourceItem);
        // set this model dirty
        setDirty();

    }

    /**
     * Delete the resource item from the entire library
     * 
     * @param resourceItem The resource item to delete
     */
    public void deleteResourceItemFromEntireLibrary(ResourceItem resourceItem)
    {
        // iterate all lists deleting resource item from each list
        for (int listIndex = 0; listIndex < getNumberOfResourceListsInMyLibrary(); listIndex++)
        {
            deleteResourceItemFromResourceListInMyLibrary(listIndex, resourceItem);
        }
        // set this model dirty
        setDirty();
    }

    /**
     * Export all the ResourceList to the file in MODS format
     * 
     * @param filename the file to export to
     * @param entry the list of resources to export
     * @throws FileNotFoundException
     * @throws MDCInternalException
     */
    public void exportAllResourcesToModsFile(String filename, ResourceList entry) throws FileNotFoundException,
            MDCInternalException
    {
        libraryDAO.exportAllResourcesToModsFile(filename, entry);
    }

    /**
     * Export all the ResourceList to the file in IMS RLI format
     * 
     * @param filename the file to export to
     * @param entry the list of resources to export
     * @throws FileNotFoundException
     * @throws MDCInternalException
     */
    public void exportAllResourcesToIMSFile(String filename, ResourceList entry) throws FileNotFoundException,
            MDCInternalException
    {
        libraryDAO.exportAllResourcesToIMSFile(filename, entry);
    }

    /**
     * Gets the list of currently supported styles by the client
     * 
     * @return A vector of styles
     * @throws MDCInternalException
     */
    public Vector getCreateDocumentStyles() throws MDCInternalException
    {
        return MDCStyle.getSupportedStyles(STYLES_XML);
    }

    /**
     * Create the document for the given style
     * 
     * @param filename the file to create document in
     * @param style The style to apply
     * @param entry the list of resources to use
     *  @param asPDF true if the output is to be converted to pdf
     * @throws FileNotFoundException
     * @throws MDCInternalException
     */
    public void applyStyle(String filename, MDCStyle style, ResourceList entry, boolean asPDF) throws FileNotFoundException,
            MDCInternalException
    {
        libraryDAO.applyStyle(filename, style, entry,asPDF);
    }
    
    /**
     * Searches MDC using the search text to return a collection of Resource Items.
     * @param searchText The string to search under
     * @return List of the resources found
     * @throws MDCInternalException
     */
    public List searchMDC(String searchText) throws MDCInternalException
    {
        return libraryDAO.searchMDC(searchText);
    }

}
