/*
 * MDC Desktop Client Created on 26-Apr-2005
 */
package org.mdcog.client.desktop.model;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import org.mdcog.client.desktop.utils.MDCConfig;
import org.mdcog.util.resolver.OpenURL;
import org.mdcog.util.transfer.ResourceTransferItem;

/**
 * This class represents a single resource item shown in the desktop client.
 * <br>
 * <br>
 * A map is used to store the key resource list data as this is subject to
 * change during the lifetime of the application. As we use object serialisation
 * this means if a new attribute is added once distributed the system will still
 * be able to load their old libraries
 */
public class ResourceItem extends MDCModel implements Serializable, Cloneable
{

    /**
     * Stores a reference to the resolver for generating an open url if not found in the configuration file
     */
    private static final String BASE_RESOLVER = "http://tdnet.bodley.ox.ac.uk/Oxford/Resolver/?";

    /**
     * As this class is serialised to disk and may have new methods added to it
     * that should not stop it loading this declerartion means that we control
     * versioning internally rather than compiler
     */
    static final long serialVersionUID = 1;

    /**
     * Use this string to get the author field
     */
    public final transient static String AUTHOR = "AUTHOR";

    /**
     * Use this string to get the type field
     */
    public final transient static String TYPE = "TYPE";

    /**
     * Use this string to get the title field
     */
    public final transient static String TITLE = "TITLE";

    /**
     * Use this string to get the abstract field
     */
    public final transient static String ABSTRACT = "ABSTRACT";

    /**
     * Use this string to get the annotation field
     */
    public final transient static String ANNOTATION = "ANNOTATION";
    
    /**
     * Use this string to get the publisher field
     */
    public final transient static String PUBLISHER = "PUBLISHER";
    
    /**
     * Use this string to get the isnn field
     */
    public final transient static String ISSN = "ISSN";
    
    /**
     * Use this string to get the isbn field
     */
    public final transient static String ISBN = "ISBN";

    /**
     * Use this string to get the web url field
     */
    public final transient static String WEB_URL = "WEB_URL";

    /**
     * Use this string to get the open url field
     */
    public final transient static String OPEN_URL = "OPEN_URL";

    /**
     * Use this string to get the local url field
     */
    public final transient static String LOCAL_URL = "LOCAL_URL";

    /**
     * Stores a reference to the originalXML
     */
    private String originalXML = "";

    /**
     * Stores a reference to the map that stores all the filed data
     */
    private HashMap fields = new HashMap();

    /**
     * Dummy method to create a list of search results
     * 
     * @return dummy list of resource items
     */
    public static List createSearchResources()
    {
        Vector tempResourceItems = new Vector();
        ResourceItem item = new ResourceItem();
        item.setAuthor("Tammy");
        item.setTitle("New Hampshire");
        item.setType("Magazine");
        item.setAbstractStr("A travellers esential advice");
        tempResourceItems.add(item);
        item = new ResourceItem();
        item.setAuthor("Garrett");
        item.setTitle("Lanscaping in the USA");
        item.setType("Book");
        item.setAbstractStr("Introduction into the landscapping business");
        tempResourceItems.add(item);
        item = new ResourceItem();
        item.setAuthor("Victor");
        item.setTitle("High Speed Motors");
        item.setType("Reference List");
        item.setAbstractStr("Top 100 of all time");
        tempResourceItems.add(item);
        item = new ResourceItem();
        item.setAuthor("Patrick");
        item.setTitle("Guiness in Ireland");
        item.setType("Book");
        item.setAbstractStr("Where to fid a good pint");
        tempResourceItems.add(item);
        item = new ResourceItem();
        item.setAuthor("Oliver");
        item.setTitle("RollerCoasters");
        item.setType("Book");
        item.setAbstractStr("Building a good coaster");
        tempResourceItems.add(item);
        item = new ResourceItem();
        item.setAuthor("Ian");
        item.setTitle("Chip shop weekly");
        item.setType("Magazine");
        item.setAbstractStr("All about fish n chips");
        tempResourceItems.add(item);
        item = new ResourceItem();
        item.setAuthor("Elly");
        item.setTitle("Dancing to Jazz");
        item.setType("Music Article");
        item.setAbstractStr("How to lead your partner");
        tempResourceItems.add(item);
        return tempResourceItems;
    }

    /**
     * Create an emty resource item
     */
    public ResourceItem()
    {
    }

    /**
     * Create a resource item extracting data from a ResourceTransferItem
     * 
     * @param transferItem the item to extract data from
     */
    public ResourceItem(ResourceTransferItem transferItem)
    {
        // set all the data extract from transfer item
        setAuthor(transferItem.getField(ResourceTransferItem.AUTHOR));
        setTitle(transferItem.getField(ResourceTransferItem.TITLE));
        setType(transferItem.getField(ResourceTransferItem.TYPE));
        setAbstractStr(transferItem.getField(ResourceTransferItem.ABSTRACT));
        setAnnotation(transferItem.getField(ResourceTransferItem.ANNOTATION));
        setWebURL(transferItem.getField(ResourceTransferItem.WEB_URL));
        setISBN(transferItem.getField(ResourceTransferItem.ISBN));
        setISSN(transferItem.getField(ResourceTransferItem.ISSN));
        setPublisher(transferItem.getField(ResourceTransferItem.PUBLISHER));

        // store the original xml that the values were extracted from
        setOriginalXML(transferItem.getRawXml());

        // mark the object clean
        setClean();
    }

    /**
     * Gets the field into the map
     * 
     * @param fieldID The field to retrieve
     * @return the object returned
     */
    private Object getField(String fieldID)
    {
        return fields.get(fieldID);
    }

    /**
     * Sets the field into the map
     * 
     * @param fieldID The field to retrieve
     * @param obj The object to store
     */
    private void setField(String fieldID, Object obj)
    {
        // make sure the key exists
        if (fields.containsKey(fieldID))
        {
            // only update if different to current value
            if (getField(fieldID).equals(obj))

            {
                // noting to update as value is the same
                return;
            }
        }
        // update the field
        fields.put(fieldID, obj);
        // set the model dirty
        setDirty();

    }

    /**
     * This method clones the orignal item and creates new copy
     * 
     * @return The cloned object
     */
    public Object clone()
    {
        ResourceItem clone = null;
        try
        {
            // this should clone all instances vaiables
            clone = (ResourceItem) super.clone();

            // As I know there are only strings stored here no need to worry
            // about deep copy
            clone.fields = (HashMap) fields.clone();
        }
        catch (CloneNotSupportedException e)
        {
            // This should never happen
            throw new InternalError(e.toString());
        }
        clone.setClean();
        return clone;
    }

    /**
     * Get original XML
     * 
     * @return Returns the originalXML.
     * 
     * @uml.property name="originalXML"
     */
    public String getOriginalXML() {
        return originalXML;
    }

    /**
     * Set original XML
     * 
     * @param originalXML The originalXML to set.
     * 
     * @uml.property name="originalXML"
     */
    public void setOriginalXML(String originalXML) {
        // only update if different
        if (!this.originalXML.equals(originalXML)) {
            this.originalXML = originalXML;
            // set the model dirty
            setDirty();
        }
    }


    /**
     * gets the abstract
     * 
     * @return Returns the abstractStr.
     */
    public String getAbstractStr()
    {
        String value = (String) getField(ABSTRACT);
        return value == null ? "" : value;
    }

    /**
     * Set the abstract
     * 
     * @param abstractStr The abstractStr to set.
     */
    public void setAbstractStr(String abstractStr)
    {
        setField(ABSTRACT, abstractStr);
    }

    /**
     * Get the author
     * 
     * @return Returns the author.
     */
    public String getAuthor()
    {
        String value = (String) getField(AUTHOR);
        return value == null ? "" : value;
    }

    /**
     * Set the author
     * 
     * @param author The author to set.
     */
    public void setAuthor(String author)
    {
        setField(AUTHOR, author);

    }

    /**
     * Get the local URL
     * 
     * @return Returns the localURL.
     */
    public String getLocalURL()
    {
        String value = (String) getField(LOCAL_URL);
        return value == null ? "" : value;
    }

    /**
     * Set the local url
     * 
     * @param localURL The localURL to set.
     */
    public void setLocalURL(String localURL)
    {
        setField(LOCAL_URL, localURL);

    }

    /**
     * Get the open URL
     * 
     * @return Returns the openURL.
     */
    public String getOpenURL()
    {
        // create the open url generator
        OpenURL myUrl = new OpenURL();
        // set base resolver and any data we store
        myUrl.setBaseResolver(MDCConfig.getInstance().getProperty(MDCConfig.BASE_RESOLVER,BASE_RESOLVER));
        myUrl.setAulast(getAuthor());
        myUrl.setTitle(getTitle());
        // generate and return URL
        return myUrl.generateOpenURL();

    }

    /**
     * Get the title
     * 
     * @return Returns the title.
     */
    public String getTitle()
    {
        String value = (String) getField(TITLE);
        return value == null ? "" : value;
    }

    /**
     * Set the title
     * 
     * @param title The title to set.
     */
    public void setTitle(String title)
    {
        setField(TITLE, title);
    }

    /**
     * Get the type
     * 
     * @return Returns the type.
     */
    public String getType()
    {
        String value = (String) getField(TYPE);
        return value == null ? "" : value;
    }

    /**
     * Set the type
     * 
     * @param type The type to set.
     */
    public void setType(String type)
    {
        setField(TYPE, type);
    }

    /**
     * Get the web URL
     * 
     * @return Returns the webURL.
     */
    public String getWebURL()
    {
        String value = (String) getField(WEB_URL);
        return value == null ? "" : value;
    }

    /**
     * Set the web URL
     * 
     * @param webURL The webURL to set.
     */
    public void setWebURL(String webURL)
    {
        setField(WEB_URL, webURL);
    }

    /**
     * Get the annotation text
     * 
     * @return Returns the annotation.
     */
    public String getAnnotation()
    {
        String value = (String) getField(ANNOTATION);
        return value == null ? "" : value;
    }

    /**
     * set the annotation text
     * 
     * @param annotation The annotation to set.
     */
    public void setAnnotation(String annotation)
    {
        setField(ANNOTATION, annotation);
    }
    
    /**
     * Get the publisher
     * 
     * @return Returns the publisher.
     */
    public String getPublisher()
    {
        String value = (String) getField(PUBLISHER);
        return value == null ? "" : value;
    }

    /**
     * Set the publisher
     * 
     * @param publisher The publisher to set.
     */
    public void setPublisher(String publisher)
    {
        setField(PUBLISHER, publisher);
    }

    /**
     * Get the isbn
     * 
     * @return Returns the isbn.
     * 
     * @uml.property name="iSBN"
     */
    public String getISBN() {
        String value = (String) getField(ISBN);
        return value == null ? "" : value;
    }

    /**
     * Set the isbn
     * 
     * @param isbn The isbn to set.
     * 
     * @uml.property name="iSBN"
     */
    public void setISBN(String isbn) {
        setField(ISBN, isbn);
    }

    /**
     * Get the issn
     * 
     * @return Returns the issn.
     * 
     * @uml.property name="iSSN"
     */
    public String getISSN() {
        String value = (String) getField(ISSN);
        return value == null ? "" : value;
    }

    /**
     * Set the issn
     * 
     * @param issn The issn to set.
     * 
     * @uml.property name="iSSN"
     */
    public void setISSN(String issn) {
        setField(ISSN, issn);
    }

}
