/*
 * MDC Desktop Client Created on 22-Apr-2005
 */
package org.mdcog.client.desktop.model;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Vector;

import javax.swing.Icon;

import org.mdcog.client.desktop.gui.images.MDCIcons;
import org.mdcog.client.desktop.utils.MDCDate;

/**
 * This class represents an entry in the resource Lists. <br>
 * <br>
 * A map is used to store the key resource list data as this is subject to
 * change during the lifetime of the application. As we use object serialisation
 * this means if a new attribute is added once distributed the system will still
 * be able to load their old libraries
 * 
 * @uml.dependency supplier="org.mdcog.client.desktop.model.ResourceItem" stereotypes="Basic::Responsibility"
 */

public class ResourceList extends MDCModel implements Cloneable, Serializable
{

    /**
     * As this class is serialised to disk and may have new methods added to it
     * that should not stop it loading this declerartion means that we control
     * versioning internally rather than compiler
     */
    static final long serialVersionUID = 1;

    /**
     * Use this string to get the author field
     */
    public final transient static String AUTHOR = "AUTHOR";

    /**
     * Use this string to get the owner field
     */
    public final transient static String OWNER = "OWNER";

    /**
     * Use this string to get the title field
     */
    public final transient static String TITLE = "TITLE";

    /**
     * Use this string to get the date created field
     */
    public final transient static String DATE_CREATED = "DATE_CREATED";

    /**
     * Use this string to get the annotation field
     */
    public final transient static String ANNOTATION = "ANNOTATION";

    /**
     * Stores a reference to the list name
     */
    private String resourceListName;

    /**
     * Stores a reference to the map that stores all the filed data
     */
    private HashMap fields = new HashMap();

    /**
     * Stores a reference to the resource items for list
     */
    private Vector resourceItems;

    /**
     * Tempory test method to create dummy records
     * 
     * @return a dummy entry
     */
    public static ResourceList createBlankAllResources()
    {
        return new ResourceList("All Resources");
    }

    /**
     * Tempory test method to create dummy ALL RESOURCES ENTRY
     * 
     * @return The All resources dummy entry
     */
    public static ResourceList createAllResources()
    {
        ResourceList entry = new ResourceList("All Resources");
        entry.setOwner("Howard");
        entry.setAuthor("Howard");
        entry.setTitle("Howards resources");
        Vector tempResourceItems = new Vector();
        ResourceItem item = new ResourceItem();
        item.setAuthor("Andy");
        item.setTitle("Golfers World");
        item.setType("Magazine");
        item.setAbstractStr("A golfers esential advice");
        item.setWebURL("www.worldgolf.com");
        tempResourceItems.add(item);
        item = new ResourceItem();
        item.setAuthor("Steve");
        item.setTitle("Birmingham City FC");
        item.setType("Book");
        item.setAbstractStr("Introduction into birminghams premier footy club");
        item.setWebURL("www.bcfc.com");
        tempResourceItems.add(item);
        item = new ResourceItem();
        item.setAuthor("Howard");
        item.setTitle("Cricket players");
        item.setType("Reference List");
        item.setAbstractStr("Top 100 of all time");
        item.setWebURL("http://www.cricinfo.com/");
        tempResourceItems.add(item);

        item = new ResourceItem();
        item.setAuthor("Bill");
        item.setTitle("Rugby rules");
        item.setType("Book");
        item.setAbstractStr("How to play rugby in 20 minutes");
        tempResourceItems.add(item);
        item = new ResourceItem();
        item.setAuthor("Stacey");
        item.setTitle("Softball in the US");
        item.setType("Book");
        item.setAbstractStr("Who knows what this game is");
        tempResourceItems.add(item);
        item = new ResourceItem();
        item.setAuthor("Tommy");
        item.setTitle("Baseball legends");
        item.setType("Hall of Fame");
        item.setAbstractStr("The best of the best");
        tempResourceItems.add(item);
        item = new ResourceItem();
        item.setAuthor("Matt");
        item.setTitle("Judo books");
        item.setType("Self Defence Article");
        item.setAbstractStr("How to not take a wooping");

        tempResourceItems.add(item);
        entry.resourceItems = tempResourceItems;
        return entry;
    }

    /**
     * Private Constructor
     */
    private ResourceList()
    {
    }

    /**
     * Checks if this model or any resource items are dirty and need saving
     * 
     * @return true if entry or any resource items are dirty
     */
    public boolean isDirty()
    {
        // is this model dirty
        if (super.isDirty())
        {
            // no need to check anymore we are dirty
            return true;
        }

        // still clean so check
        for (int i = 0; i < resourceItems.size(); i++)
        {
            // check each resource list item to see if it is dirty
            if (((ResourceItem) resourceItems.get(i)).isDirty())
            {
                //  no need to check anymore we are dirty
                return true;
            }
        }
        return false;
    }

    /**
     * Sets the model clean and cleans all ResourceItems
     */
    public void setClean()
    {
        super.setClean();
        // clean all ResourceItem's
        for (int i = 0; i < resourceItems.size(); i++)
        {
            ((ResourceItem) resourceItems.get(i)).setClean();
        }
    }

    /**
     * Gets the field into the map
     * 
     * @param fieldID The field to retrieve
     * @return the object returned
     */
    private Object getField(String fieldID)
    {
        return fields.get(fieldID);
    }

    /**
     * Sets the field into the map
     * 
     * @param fieldID The field to retrieve
     * @param obj The object to store
     */
    private void setField(String fieldID, Object obj)
    {
        // make sure the key exists
        if (fields.containsKey(fieldID))
        {
            // only update if different to current value
            if (getField(fieldID).equals(obj))

            {
                // noting to update as value is the same
                return;
            }
        }
        // update the field
        fields.put(fieldID, obj);
        // set the model dirty
        setDirty();

    }

    /**
     * This method clones the orignal item and creates new copy
     * 
     * @return The copied object
     */
    public Object clone()
    {
        ResourceList clone = null;
        ResourceItem itemClone = null;
        try
        {
            // this should clone all instances vaiables
            clone = (ResourceList) super.clone();
            // clone the resource items list
            clone.resourceItems = (Vector) this.resourceItems.clone();
            // need to deep copy the resource items as cloneing vector will only
            // shallow copy
            // empty cloned vector
            clone.resourceItems.clear();
            // loop copying original items. We could use serialisatio to do deep
            // copy but as
            // the structure is not that complex faster to do it this way
            for (int i = 0; i < this.resourceItems.size(); i++)
            {
                // add clone of originals resource item into
                // ResourceItem resource items
                itemClone = (ResourceItem) ((ResourceItem) this.resourceItems.get(i)).clone();
                clone.addNewResourceItem(itemClone);
            }

            // As I know there are only strings stored here no need to worry
            // about deep copy
            clone.fields = (HashMap) fields.clone();
        }
        catch (CloneNotSupportedException e)
        {
            // This should never happen
            throw new InternalError(e.toString());
        }
        // set the clone clean
        clone.setClean();
        return clone;
    }

    /**
     * Returns a boolean if not enough data is stored in this resource list to
     * export it or create a list from it
     * 
     * @return True if have author,owner,description and at least one resource
     *         item
     */
    public boolean canCreateList()
    {
        // must enter author,owner,description and have at least one
        // resourceitem to export
        if ((getAuthor().length() > 0) && (getOwner().length() > 0) && (getTitle().length() > 0)
                && (getResourceItems().size() > 0))
        {
            return true;
        }
        return false;
    }

    /**
     * Construct a new ResourceListEntry
     * 
     * @param resourceListName The name of the entry
     */
    public ResourceList(String resourceListName)
    {
        setCreationDate(MDCDate.getCurrentDate());
        this.resourceListName = resourceListName;
        this.resourceItems = new Vector();

    }

    /**
     * Add the new resource item to the list as long as not already in list
     * 
     * @param item The item to add
     */
    public void addNewResourceItem(ResourceItem item)
    {
        // only add item if it does not already exist in list
        if (!resourceItems.contains(item))
        {
            resourceItems.add(item);
            // set the model dirty
            setDirty();
        }
    }

    /**
     * Delete the resource item from the list
     * 
     * @param item The item to delete
     */
    public void deleteResourceItem(ResourceItem item)
    {
        resourceItems.remove(item);
        // set the model dirty
        setDirty();
    }

    /**
     * @return Returns the resourceItems.
     * 
     * @uml.property name="resourceItems"
     */
    public List getResourceItems() {
        return resourceItems;
    }

    /**
     * Set the resource list name
     * 
     * @param resourceListName
     * 
     * @uml.property name="resourceListName"
     */
    public void setResourceListName(String resourceListName) {
        this.resourceListName = resourceListName;
        //set the model dirty
        setDirty();
    }

    /**
     * Return the resource list name
     * 
     * @return The resource list name
     * 
     * @uml.property name="resourceListName"
     */
    public String getResourceListName() {
        return resourceListName;
    }

    /**
     * Return the image icon with entry
     * 
     * @return The image for this resource list
     */
    public Icon getImage()
    {
        return MDCIcons.RESOURCELIST_ICON;
    }

    /**
     * Overload toString to return just the resource list name
     * 
     * @return String representation of entry
     */
    public String toString()
    {
        return resourceListName;
    }

    /**
     * Get the author
     * 
     * @return Returns the author.
     */
    public String getAuthor()
    {
        String value = (String) getField(AUTHOR);
        return value == null ? "" : value;
    }

    /**
     * Set the author
     * 
     * @param author The author to set.
     */
    public void setAuthor(String author)
    {
        setField(AUTHOR, author);
    }

    /**
     * Get the title
     * 
     * @return Returns the title.
     */
    public String getTitle()
    {
        String value = (String) getField(TITLE);
        return value == null ? "" : value;
    }

    /**
     * Set the title
     * 
     * @param title The title to set.
     */
    public void setTitle(String title)
    {
        setField(TITLE, title);
    }

    /**
     * Get the owner
     * 
     * @return Returns the owner.
     */
    public String getOwner()
    {
        String value = (String) getField(OWNER);
        return value == null ? "" : value;
    }

    /**
     * Set the owner
     * 
     * @param owner The owner to set.
     */
    public void setOwner(String owner)
    {
        setField(OWNER, owner);
    }

    /**
     * Gets the annotation
     * 
     * @return Returns the annotation.
     */
    public String getAnnotation()
    {
        String value = (String) getField(ANNOTATION);
        return value == null ? "" : value;
    }

    /**
     * Sets the annotation
     * 
     * @param annotation The annotation to set.
     */
    public void setAnnotation(String annotation)
    {
        setField(ANNOTATION, annotation);
    }

    /**
     * Gets the creation date
     * 
     * @return Returns the date created.
     */
    public String getCreationDate()
    {
        String value = (String) getField(DATE_CREATED);
        return value == null ? "" : value;
    }

    /**
     * Sets the creation date
     * 
     * @param dateCreated The creation date to set.
     */
    public void setCreationDate(String dateCreated)
    {
        setField(DATE_CREATED, dateCreated);
    }
}
