/*
 * MDC Desktop Client Created on 13-Jun-2005
 */
package org.mdcog.client.styles;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.Vector;

import javax.xml.transform.Result;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.sax.SAXResult;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

import org.apache.avalon.framework.logger.NullLogger;
import org.apache.fop.apps.Driver;
import org.apache.fop.messaging.MessageHandler;
import org.mdcog.client.desktop.gui.error.MDCInternalException;
import org.mdcog.util.DomUtils;
import org.mdcog.util.DomUtilsException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

/**
 * This class represents a style that can be applied to a resource list
 */
public class MDCStyle
{

    /**
     * Stores a reference to the name of this style
     */
    private static final String FO_CONVERTER = "org/mdcog/client/styles/xhtml2fo.xsl";

    /**
     * Stores a reference to the FOP driver for PDF rendering
     */
    private Driver driver = null;

    /**
     * Stores a reference to the name of this style
     */
    private String name = "Undefined";

    /**
     * Stores a reference to the style template name to load from resources
     */
    private String styleTemplateResourceLookup = "";

    /**
     * This method looks up the XML file from the resources and loads all the
     * defined styles.
     * 
     * @param configResourceLocation location of the xml file in the resources
     * @return Vector configured with supportted styles
     * @throws MDCInternalException
     */
    public static Vector getSupportedStyles(String configResourceLocation) throws MDCInternalException
    {
        Vector styles = new Vector();
        // get an input stream on the xml file
        InputStream inStream = MDCStyle.class.getClassLoader().getResourceAsStream(configResourceLocation);
        if (inStream == null)
        {
            throw new MDCInternalException("Unable to load styles resource " + configResourceLocation);
        }

        try
        {
            StringBuffer xml = new StringBuffer();

            // open the file and extract the string
            BufferedReader buffReader = new BufferedReader(new InputStreamReader(inStream));
            String str;
            // loop round reading each line into buffer
            while ((str = buffReader.readLine()) != null)
            {
                xml.append(str);
            }
            // close the reader
            buffReader.close();

            // parse the XML in to the DOM
            Document doc = DomUtils.parse(xml.toString());

            // extract the styles collection out of the dom
            NodeList root = doc.getChildNodes();
            // Get the list of nodes under the first node in the tree
            // ie All the SYTLE nodes under STYLES
            NodeList list = root.item(0).getChildNodes();
            // Process each STYLE node
            for (int i = 0; i < list.getLength(); i++)
            {
                // Get the node as an element
                Element node = (Element) list.item(i);
                // get the text of the name attribute
                String name = node.getAttribute("name");
                // get the text value for the style which is the template
                // location
                String value = DomUtils.getTextValue(node);
                // check these values are vaid before creating style
                if (name.length() == 0 || value.length() == 0)
                {
                    throw new MDCInternalException("Invalid style format");
                }
                // add the new style to the collection
                styles.add(new MDCStyle(name, value));
            }

        }
        catch (IOException e)
        {
            e.printStackTrace();
            throw new MDCInternalException("Unable to load styles resource ");
        }
        catch (DomUtilsException e)
        {
            e.printStackTrace();
            throw new MDCInternalException("Unable to parse styles resource ");
        }
        return styles;
    }

    /**
     * Constructor of the style setting its name
     * 
     * @param name The name of the style
     * @param templateLookup The name to lookup in resources to find style
     *        template
     */
    public MDCStyle(String name, String templateLookup)
    {
        this.name = name;
        styleTemplateResourceLookup = templateLookup;
        // Construct FOP driver
        driver = new Driver();
        driver.setLogger(new NullLogger());
        MessageHandler.setScreenLogger(new NullLogger());
    }

    /**
     * Returns the name of this style. Used to display in Combo boxes
     * 
     * @return The name of the style
     */
    public String toString()
    {
        return name;
    }

    /**
     * Applies this style to the xml
     * 
     * @param xml The mods xml to apply the style to
     * @return HTML is returned for the applied style
     * @throws MDCInternalException
     */
    public String applyStyle(String xml) throws MDCInternalException
    {
        // get the teansformer factory
        TransformerFactory tFactory = TransformerFactory.newInstance();

        // try and load the resource for this style
        InputStream inStream = this.getClass().getClassLoader().getResourceAsStream(styleTemplateResourceLookup);
        if (inStream == null)
        {
            throw new MDCInternalException("Unable to load resource " + styleTemplateResourceLookup);
        }

        // create a stream source on the XSLT style
        StreamSource xslt = new StreamSource(inStream);

        // create a stream source on the supplied XML
        StreamSource source = new StreamSource(new StringReader(xml));
        // create a writer to write rersult to
        StringWriter writer = new StringWriter();
        try
        {
            // get a new transformer
            Transformer transformer = tFactory.newTransformer(xslt);
            // set the writer
            StreamResult streamResult = new StreamResult(writer);
            // perform transform
            transformer.transform(source, streamResult);
        }
        catch (Exception e)
        {
            e.printStackTrace();
            throw new MDCInternalException("Error Applying Style - Possible stylesheet problem");
        }
        //flush and return string
        writer.flush();
        return writer.toString();
    }

    /**
     * This converts the applied style to PDF
     * 
     * @param xHTML the html to convert
     * @param outStream The stream to write pdf to
     * @return The converted html as a PDF
     * @throws MDCInternalException
     */
    public void toPDF(String xHTML, OutputStream outStream) throws MDCInternalException
    {

        // try and load the FO convert style
        InputStream inStream = this.getClass().getClassLoader().getResourceAsStream(FO_CONVERTER);
        if (inStream == null)
        {
            throw new MDCInternalException("Unable to load resource " + FO_CONVERTER);
        }
        // create a stream source on the XSLT style
        StreamSource xslt = new StreamSource(inStream);

        try
        {
            // Setup Transformer
            TransformerFactory factory = TransformerFactory.newInstance();
            Transformer transformer = factory.newTransformer(xslt);

            // Reset the FOP driver for next conversion
            driver.reset();

            // Setup Renderer (output format)
            driver.setRenderer(Driver.RENDER_PDF);
            driver.setOutputStream(outStream);

            // create a stream source on the supplied XHTML
            StreamSource source = new StreamSource(new StringReader(xHTML));

            // Resulting SAX events (the generated FO) must be piped through FOP
            Result res = new SAXResult(driver.getContentHandler());

            // Start XSLT transformation and FOP processing
            transformer.transform(source, res);
        }
        catch (Exception e)
        {
            e.printStackTrace();
            throw new MDCInternalException("Error Converting to PDF");
        }

    }

}
