/*
 * MDC Desktop Client Created on 24-May-2005
 */
package org.mdcog.util.transfer;

import java.util.ArrayList;
import java.util.Iterator;

import org.mdcog.util.DomUtils;
import org.mdcog.util.DomUtilsException;
import org.mdcog.util.ModsUtils;
import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * This class handles the MDC client and sever tranferer format.
 */
public class MDCXMLDataTransferer implements MDCDataTransferer
{

    /**
     * Stores a reference to the ResourceTransferItems extracted or to be
     * exported
     */
    private ArrayList items = new ArrayList();

    /**
     * Stores a reference to the ResourceTransferHeader for the list imported or
     * to be exported
     * 
     * @uml.property name="header"
     * @uml.associationEnd multiplicity="(0 1)"
     */
    private ResourceTransferHeader header = null;


    /**
     * This method creates a new list blanking any currently loaded data
     * 
     * @param header The header to create the list about. If Null a blank header
     *        will be created
     */
    public void createEmptyList(ResourceTransferHeader header)
    {
        // assign the header and empty the array list
        setHeader(header);
        items.clear();
    }

    /**
     * This method loads the list from XML supplied, blanking any currently
     * loaded data
     * 
     * @param transferData String representation of the data.
     * @throws TransferException thrown on any transfer problem encaspulating
     *         exception that caused problem
     */
    public void loadList(String transferData) throws TransferException
    {
        try
        {
            // parse the transfer data XML string into a DOM document
            Document doc = DomUtils.parse(transferData);
            Element modsroot = doc.getDocumentElement();
            // get the list of mod elements
            NodeList list = modsroot.getElementsByTagName("mods");
            // make sure we found at least one mods element else the file is not
            // valid
            if (list.getLength() < 1)
            {
                throw new TransferException("No mods tags found in the import xml");
            }
            // first mods tag is the header always defined to be the resource
            // list header so create a header object and ask it to initalise
            // from the raw xml
            header = new ResourceTransferHeader();
            header.setRawXml((Node) list.item(0));
            // clear any current items as we are starting a new import
            items.clear();
            // all the rest of the mods elements are actual resource items
            for (int i = 1; i < list.getLength(); i++)
            {
                // Extract the element and store it into the resource transfer
                // item
                Node node = (Node) list.item(i);
                ResourceTransferItem resitem = new ResourceTransferItem();
                resitem.setRawXml(node);
                // add the newly created item into the list
                items.add(resitem);
            }
        }
        catch (Exception e)
        {
            throw new TransferException(e);
        }
    }

    /**
     * Converts the current list into MODS format. This method works on the
     * rawxml and assumes any updates to that rawxml has already been done
     * 
     * @return The list as XML in MODS format
     * @throws TransferException thrown on any transfer problem encaspulating
     *         exception that caused problem
     */
    public String toMODS() throws TransferException
    {
        try
        {
            if (header == null || items == null)
            {
                throw new TransferException("Not enough information to convert");
            }

            // create a new document
            Document doc = DomUtils.newDocument();
            // add the mods collection element
            Node modsCollection = ModsUtils.addModCollection(doc);
            // add header to mods collection element
            modsCollection.appendChild(doc.importNode(header.getRawXmlNode(),true));

            ResourceTransferData item = null;
            Iterator itr = items.iterator();

            // loop round resource items using iterator
            while (itr.hasNext())
            {
                item = (ResourceTransferData) itr.next();
                modsCollection.appendChild(doc.importNode(item.getRawXmlNode(),true));
            }
            // convert the now completed DOM into a string
            return DomUtils.getNodeAsString(doc);
        }
        catch (DOMException e)
        {
            e.printStackTrace();
            throw new TransferException(e);
        }
        catch (DomUtilsException e)
        {
            e.printStackTrace();
            throw new TransferException(e);
        }
    }

    /**
     * Converts the current list into RLI format. This method works on the
     * rawxml and assumes any updates to that rawxml has already been done
     * 
     * @return The list as XML in RLI format
     * @throws TransferException thrown on any transfer problem encaspulating
     *         exception that caused problem
     */
    public String toRLI() throws TransferException
    {
        if (header == null || items == null)
        {
            throw new TransferException("Not enough information to convert");
        }
        // should do straight conversion on raw xml
        return "";
    }

    /**
     * This method extracts all the header information about the list. The
     * resourceHeader returned from this method will have already been parsed
     * and populated with all the extracted data
     * 
     * @return The Header object
     * 
     * @uml.property name="header"
     */
    public ResourceTransferHeader getHeader() {
        // Method needs to store raw XML and then extract all the fields into
        // the map before being returned from here
        return header;
    }


    /**
     * This method provides an iterator round each of the resource items in the
     * list. Each element returned by the iterator will be of type Resource
     * 
     * @return An iterator of all the Resource items
     */
    public Iterator getResourceIterator()
    {
        // The Resource Items returned by iterator must already have extracted
        // all the fields into the map.
        return items.iterator();
    }

    /**
     * This method adds a resource to the list.
     * 
     * @param resource The Resource to add
     */
    public void addResource(ResourceTransferItem resource)
    {
        items.add(resource);
    }

    /**
     * This method sets the current ResourceTransferHeader.
     * 
     * @param header The header to be set
     * 
     * @uml.property name="header"
     */
    public void setHeader(ResourceTransferHeader header) {
        this.header = header;
    }

}
