package org.mdcog.util.transfer;

import java.util.HashMap;
import java.util.Map;

import org.mdcog.util.DomUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * This is the superclass
 */
/**
 * 
 */
public abstract class ResourceTransferData
{

    /**
     * Stores a reference to the fields extracted
     */
    private Map fields = new HashMap();

    /**
     * Stores a reference to the doucment ID for synchronizing with exist
     */
    private String docId = "";

    /**
     * Stores a reference to the doucment source for synchronizing with exist
     */
    private String docSource = "";

    /**
     * Stores a reference to the raw xml that represents this transfer data
     */
    protected String rawXml = "";
    
    /**
     * Stores a reference to the raw xml node representing this transfer data item
     */
    protected Node rawXmlNode = null;

    /**
     * Protected constructor to force user to use base classes
     */
    protected ResourceTransferData()
    {
        // stop anyone creating this class directly
    }

    /**
     * Sets the document ID for synchronizing with exist
     * 
     * @param docId The docId to set.
     * 
     * @uml.property name="docId"
     */
    public void setDocId(String docId) {
        this.docId = docId;
    }

    /**
     * Gets the document ID for synchronizing with exist
     * 
     * @return Returns the docId.
     * 
     * @uml.property name="docId"
     */
    public String getDocId() {
        if (docId == null) {
            docId = "";
        }
        return docId;
    }

    /**
     * Sets the document Source for synchronizing with exist
     * 
     * @param docSource The docSource to set.
     * 
     * @uml.property name="docSource"
     */
    public void setDocSource(String docSource) {
        this.docSource = docSource;
    }

    /**
     * Gets the document Source for synchronizing with exist
     * 
     * @return Returns the docSource.
     * 
     * @uml.property name="docSource"
     */
    public String getDocSource() {
        if (docSource == null) {
            docSource = "";
        }
        return docSource;
    }

    /**
     * Gets all the extract fields from the raw XML
     * 
     * @return Returns the fields.
     * 
     * @uml.property name="fields"
     */
    public Map getFields() {
        if (fields == null) {
            fields = new HashMap();
        }
        return fields;
    }

    /**
     * Sets all the extract fields from the raw XML
     * 
     * @param fields The fields to set.
     * 
     * @uml.property name="fields"
     */
    public void setFields(Map fields) {
        this.fields = fields;
    }


    /**
     * Sets a specific field
     * 
     * @param fieldname The fieldname of field to set.
     * @param value The value of field to set.
     */
    public void setField(String fieldname, String value)
    {
        if (value == null)
        {
            fields.put(fieldname, "");
        }
        else
        {
            fields.put(fieldname, value);
        }
    }

    /**
     * Gets a specific field
     * 
     * @param fieldname The fieldname of field to get.
     * @return Returns the field.
     */
    public String getField(String fieldname)
    {
        String str = (String) fields.get(fieldname);
        if (str == null)
        {
            return "";
        }
        else
        {
            return str;
        }
    }

    /**
     * Gets the raw XML that relates to the fields in this header
     * 
     * @return Returns the rawXml.
     * 
     * @uml.property name="rawXml"
     */
    public String getRawXml() {
        if (rawXml == null) {
            return "";
        }
        return rawXml;
    }

    /**
     * Returns a reference to the raw node representing the currently loaded xml
     * @return the raw node representing the currently loaded xml
     * 
     * @uml.property name="rawXmlNode"
     */
    public Node getRawXmlNode() {
        return rawXmlNode;
    }

    /**
     * Sets the raw XML that constitutes this header. The XML is parsed and
     * passed to setRawXml(rawXmlAsDomNode) to be processed and stored.
     * 
     * @param rawXml The rawxml to set.
     * @throws TransferException thrown on any transfer problem encaspulating
     *         exception that caused problem
     * 
     * @uml.property name="rawXml"
     */
    public void setRawXml(String rawXml) throws TransferException {
        try {
            // Parse the xml
            Document doc = DomUtils.parse(rawXml);
            // store the raw node fo later use
            rawXmlNode = doc.getDocumentElement();
            setRawXml(rawXmlNode);
        } catch (Exception e) {
            throw new TransferException(e);
        }
    }

    /**
     * Sets the raw XML that constitutes this header. Must be overridden by a
     * sub class. When this method is called the xml should be parsed and the
     * fields map initialised from appropriate fields in the xml document. You
     * should call super(rawXmlAsDomNode), so that the source xml string is
     * correctly stored.
     * 
     * @param rawXmlAsDomNode The rawxml to set.
     * @throws TransferException thrown on any transfer problem encaspulating
     *         exception that caused problem
     */
    protected void setRawXml(Node rawXmlAsDomNode) throws TransferException
    {
        // Store the node as a string.
        try
        {
            rawXml = DomUtils.getNodeAsString(rawXmlAsDomNode);
        }
        catch (Exception e)
        {
            throw new TransferException(e);
        }
    }

    /**
     * This method updates the raw xml with the current data stored in the
     * fields map. It is abstract and every specific Transfer Data class must
     * implement it to convert their specific data. If the raw xml is blank then
     * this method will also create the xml from scratch. 
     * 
     * @throws TransferException thrown on any transfer problem encaspulating
     *         exception that caused problem
     */
    public abstract void updateRawXMLFromFields() throws TransferException;

}
